@@ -72,20 +72,30 @@ const char user_keywords[] = // Definition of own keywords, not used by MySQL.
   [mEditor setAutoresizingMask: NSViewWidthSizable | NSViewHeightSizable];
   
   // Let's load some text for the editor, as initial content.
-  NSError* error = nil;
-  
-  NSString* path = [[NSBundle mainBundle] pathForResource: @"TestData" 
-                                                   ofType: @"sql" inDirectory: nil];
-  
-  NSString* sql = [NSString stringWithContentsOfFile: path
-                                            encoding: NSUTF8StringEncoding
-                                               error: &error];
-  if (error && [[error domain] isEqual: NSCocoaErrorDomain])
-    NSLog(@"%@", error);
-  
+  NSString *sql = [self exampleText];
+
   [mEditor setString: sql];
 
   [self setupEditor];
+  
+  sciExtra = nil;
+}
+
+- (NSString *) exampleText
+{
+  NSError* error = nil;
+
+  NSString* path = [[NSBundle mainBundle] pathForResource: @"TestData"
+                                                   ofType: @"sql" inDirectory: nil];
+
+  NSString *sql = [NSString stringWithContentsOfFile: path
+                                            encoding: NSUTF8StringEncoding
+                                               error: &error];
+
+  if (error && [[error domain] isEqual: NSCocoaErrorDomain])
+    NSLog(@"%@", error);
+  
+  return sql;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -271,6 +281,26 @@ static const char * box_xpm[] = {
     [self showAutocompletion];
 }
 
+- (IBAction) addRemoveExtra: (id) sender
+{
+	if (sciExtra) {
+		[sciExtra removeFromSuperview];
+		sciExtra = nil;
+	} else {
+		NSRect newFrame = mEditHost.frame;
+		newFrame.origin.x += newFrame.size.width + 5;
+		newFrame.origin.y += 46;
+		newFrame.size.width = 96;
+		newFrame.size.height -= 60;
+
+		sciExtra = [[[ScintillaView alloc] initWithFrame: newFrame] autorelease];
+		[[[mEditHost window]contentView] addSubview: sciExtra];
+		[sciExtra setGeneralProperty: SCI_SETWRAPMODE parameter: SC_WRAP_WORD value: 1];
+		NSString *sql = [self exampleText];
+		[sciExtra setString: sql];
+	}
+}
+
 -(IBAction) setFontQuality: (id) sender
 {
     [ScintillaView directCall:mEditor message:SCI_SETFONTQUALITY wParam:[sender tag] lParam:0];
