@@ -47,6 +47,7 @@
 #include <vector>
 #include <set>
 #include <map>
+#include <memory>
 
 #include <gtk/gtk.h>
 
@@ -173,7 +174,7 @@ static gboolean ReceiverPipeSignal(GIOChannel *source, GIOCondition condition, v
 			status = g_io_channel_read_chars(source, pipeData,
 			        sizeof(pipeData) - 1, &readLength, &error);
 		}
-		DirectorExtension *ext = reinterpret_cast<DirectorExtension *>(data);
+		DirectorExtension *ext = static_cast<DirectorExtension *>(data);
 		ext->HandleStringMessage(pipeString.c_str());
 	}
 #ifndef GDK_VERSION_3_6
@@ -385,7 +386,7 @@ void DirectorExtension::HandleStringMessage(const char *message) {
 			// if there are no more slots, then the returned pipename is '*'
 			char pipeName[MAX_PATH];
 			if (! SendPipeAvailable()) {
-				strcpy(pipeName,"*");
+				StringCopy(pipeName,"*");
 			} else {
 				sprintf(pipeName,"%s/SciTE.%d.%d.out", g_get_tmp_dir(), getpid(), kount++);
 			}
