@@ -8,14 +8,37 @@
 
 // http://www.microsoft.com/msj/0797/win320797.aspx
 
+#include <glib.h>
+
 #include "Mutex.h"
 
 class GTKMutex : public Mutex {
 private:
-	virtual void Lock() {}
-	virtual void Unlock() {}
-	GTKMutex() {}
-	virtual ~GTKMutex() {}
+#if GLIB_CHECK_VERSION(2,31,0)
+	GMutex m;
+#endif
+	GMutex *pm;
+	void Lock() override {
+		g_mutex_lock(pm);
+	}
+	void Unlock() override {
+		g_mutex_unlock(pm);
+	}
+	GTKMutex() {
+#if GLIB_CHECK_VERSION(2,31,0)
+		pm = &m;
+		g_mutex_init(pm);
+#else
+		pm = g_mutex_new();
+#endif
+	}
+	~GTKMutex() override {
+#if GLIB_CHECK_VERSION(2,31,0)
+		g_mutex_clear(pm);
+#else
+		g_mutex_free(pm);
+#endif
+	}
 	friend class Mutex;
 };
 
