@@ -150,8 +150,8 @@ void WComboBoxEntry::ClearList() {
 
 void WComboBoxEntry::FillFromMemory(const std::vector<std::string> &mem, bool useTop) {
 	ClearList();
-	for (size_t i = 0; i < mem.size(); i++) {
-		AppendText(mem[i].c_str());
+	for (const std::string &s : mem) {
+		AppendText(s.c_str());
 	}
 	if (useTop) {
 		gtk_entry_set_text(Entry(), mem[0].c_str());
@@ -207,7 +207,7 @@ static void GreyToAlpha(GdkPixbuf *ppb, GdkColor fore) {
 	}
 }
 
-void WCheckDraw::Create(const char **xpmImage, GUI::gui_string toolTip, GtkStyle *pStyle_) {
+void WCheckDraw::Create(const char **xpmImage, const GUI::gui_string &toolTip, GtkStyle *pStyle_) {
 	GtkWidget *button = gtk_toggle_button_new();
 	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), FALSE);
