@@ -26,6 +26,7 @@
 #include "StyleContext.h"
 #include "CharacterSet.h"
 #include "LexerModule.h"
+#include "DefaultLexer.h"
 #include "LexerBase.h"
 
 #ifdef SCI_NAMESPACE
@@ -42,7 +43,7 @@ struct latexFoldSave {
 		for (int i = 0; i < 8; ++i) openBegins[i] = save.openBegins[i];
 	}
 	int openBegins[8];
-	int structLev;
+	Sci_Position structLev;
 };
 
 class LexerLaTeX : public LexerBase {
@@ -60,7 +61,7 @@ private:
 		if (static_cast<Sci_Position>(modes.size()) > numLines * 2 + 256)
 			modes.resize(numLines + 128);
 	}
-	
+
 	vector<latexFoldSave> saves;
 	void setSave(Sci_Position line, const latexFoldSave &save) {
 		if (line >= static_cast<Sci_Position>(saves.size())) saves.resize(line + 1);
@@ -78,11 +79,11 @@ private:
 			saves.resize(numLines + 128);
 	}
 public:
-	static ILexer *LexerFactoryLaTeX() {
+	static ILexer4 *LexerFactoryLaTeX() {
 		return new LexerLaTeX();
 	}
-	void SCI_METHOD Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, IDocument *pAccess);
-	void SCI_METHOD Fold(Sci_PositionU startPos, Sci_Position length, int initStyle, IDocument *pAccess);
+	void SCI_METHOD Lex(Sci_PositionU startPos, Sci_Position length, int initStyle, IDocument *pAccess) override;
+	void SCI_METHOD Fold(Sci_PositionU startPos, Sci_Position length, int initStyle, IDocument *pAccess) override;
 };
 
 static bool latexIsSpecial(int ch) {
@@ -192,7 +193,7 @@ void SCI_METHOD LexerLaTeX::Lex(Sci_PositionU startPos, Sci_Position length, int
 	int state = initStyle;
 	if (state == SCE_L_ERROR || state == SCE_L_SHORTCMD || state == SCE_L_SPECIAL)   // should not happen
 		latexStateReset(mode, state);
-	
+
 	char chNext = styler.SafeGetCharAt(startPos);
 	char chVerbatimDelim = '\0';
 	styler.StartSegment(startPos);
@@ -207,7 +208,7 @@ void SCI_METHOD LexerLaTeX::Lex(Sci_PositionU startPos, Sci_Position length, int
 			chNext = styler.SafeGetCharAt(i + 1);
 			continue;
 		}
-		
+
 		if (ch == '\r' || ch == '\n')
 			setMode(styler.GetLine(i), mode);
 
@@ -256,7 +257,7 @@ void SCI_METHOD LexerLaTeX::Lex(Sci_PositionU startPos, Sci_Position length, int
 				state = SCE_L_COMMENT;
 				break;
 			}
-			break;	
+			break;
 		// These 3 will never be reached.
 		case SCE_L_ERROR:
 		case SCE_L_SPECIAL:
@@ -400,7 +401,7 @@ void SCI_METHOD LexerLaTeX::Lex(Sci_PositionU startPos, Sci_Position length, int
 					chNext = styler.SafeGetCharAt(i + 1);
 					mode = 0;
 					state = SCE_L_DEFAULT;
-				} else { // This may not be an error, e.g. \begin{equation}\text{$a$}\end{equation}	
+				} else { // This may not be an error, e.g. \begin{equation}\text{$a$}\end{equation}
 					styler.ColourTo(i, SCE_L_SHORTCMD);
 				}
 				break;
