@@ -55,7 +55,7 @@ static inline bool IsANumberChar(int ch) {
  */
 static void CheckForKeyword(StyleContext& sc, WordList* keywordlists[], int activeState)
 {
-  int length = sc.LengthCurrent() + 1; // +1 for the next char
+  Sci_Position length = sc.LengthCurrent() + 1; // +1 for the next char
   char* s = new char[length];
   sc.GetCurrentLowered(s, length);
   if (keywordlists[0]->InList(s))
@@ -267,10 +267,13 @@ static void ColouriseMySQLDoc(Sci_PositionU startPos, Sci_Position length, int i
               {
                 sc.SetState(SCE_MYSQL_COMMENT | activeState);
 
-                // Skip comment introducer and check for hidden command.
-                sc.Forward(2);
-                if (sc.ch == '!')
+                // Skip first char of comment introducer and check for hidden command.
+                // The second char is skipped by the outer loop.
+                sc.Forward();
+                if (sc.GetRelativeCharacter(1) == '!')
                 {
+                  // Version comment found. Skip * now.
+                  sc.Forward();
                   activeState = HIDDENCOMMAND_STATE;
                   sc.ChangeState(SCE_MYSQL_HIDDENCOMMAND);
                 }
@@ -328,7 +331,7 @@ static bool IsStreamCommentStyle(int style)
  * Code copied from StyleContext and modified to work here. Should go into Accessor as a
  * companion to Match()...
  */
-bool MatchIgnoreCase(Accessor &styler, Sci_Position currentPos, const char *s)
+static bool MatchIgnoreCase(Accessor &styler, Sci_Position currentPos, const char *s)
 {
   for (Sci_Position n = 0; *s; n++)
   {
@@ -359,7 +362,7 @@ static void FoldMySQLDoc(Sci_PositionU startPos, Sci_Position length, int initSt
 	int styleNext = styler.StyleAt(startPos);
 	int style = initStyle;
   int activeState = (style == SCE_MYSQL_HIDDENCOMMAND) ? HIDDENCOMMAND_STATE : style & HIDDENCOMMAND_STATE;
-	
+
   bool endPending = false;
 	bool whenPending = false;
 	bool elseIfPending = false;
@@ -372,11 +375,11 @@ static void FoldMySQLDoc(Sci_PositionU startPos, Sci_Position length, int initSt
 		style = styleNext;
 		styleNext = styler.StyleAt(i + 1);
     activeState = (style == SCE_MYSQL_HIDDENCOMMAND) ? HIDDENCOMMAND_STATE : style & HIDDENCOMMAND_STATE;
-    
+
     char currentChar = nextChar;
     nextChar = styler.SafeGetCharAt(i + 1);
 		bool atEOL = (currentChar == '\r' && nextChar != '\n') || (currentChar == '\n');
-	
+
     switch (MASKACTIVE(style))
     {
       case SCE_MYSQL_COMMENT:
@@ -389,7 +392,7 @@ static void FoldMySQLDoc(Sci_PositionU startPos, Sci_Position length, int initSt
         break;
       case SCE_MYSQL_COMMENTLINE:
         if (foldComment)
-        { 
+        {
           // Not really a standard, but we add support for single line comments
           // with special curly braces syntax as foldable comments too.
           // MySQL needs -- comments to be followed by space or control char
@@ -497,12 +500,12 @@ static void FoldMySQLDoc(Sci_PositionU startPos, Sci_Position length, int initSt
                 }
               }
             }
-          
+
           // Keep the current end state for the next round.
           endPending = endFound;
         }
         break;
-        
+
       default:
         if (!isspacechar(currentChar) && endPending)
         {
@@ -544,7 +547,7 @@ static void FoldMySQLDoc(Sci_PositionU startPos, Sci_Position length, int initSt
         lev |= SC_FOLDLEVELHEADERFLAG;
       if (lev != styler.LevelAt(lineCurrent))
         styler.SetLevel(lineCurrent, lev);
-      
+
       lineCurrent++;
       levelCurrent = levelNext;
       visibleChars = 0;
