@@ -205,12 +205,12 @@ static void ColouriseOScriptDoc(Sci_PositionU startPos, Sci_Position length,
 	bool isFirstToken = true;
 	// It starts with true at the beginning of a line and changes to false as
 	// soon as the first identifier on the line is passed by.
-	bool isFirstIdentifier = true; 
+	bool isFirstIdentifier = true;
 	// It becomes false when #ifdef DOC (the preprocessor directive often
 	// used to start a documentation comment) is encountered and remain false
 	// until the end of the documentation block is not detected. This is done
 	// by checking for the complementary #endif preprocessor directive.
-	bool endDocComment = false; 
+	bool endDocComment = false;
 
 	for (; sc.More(); sc.Forward()) {
 
@@ -464,7 +464,7 @@ static void FoldOScriptDoc(Sci_PositionU startPos, Sci_Position length, int init
 	char chNext = styler[startPos];
 	int styleNext = styler.StyleAt(startPos);
 	int style = initStyle;
-	int lastStart = 0;
+	Sci_Position lastStart = 0;
 
 	for (Sci_Position i = startPos; i < endPos; i++) {
 		char ch = chNext;
