@@ -5,12 +5,8 @@
 // Copyright 1998-2010 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-#include <ctype.h>
-#include <stdarg.h>
-#include <assert.h>
+#include <cstdlib>
+#include <cassert>
 
 #include <string>
 
@@ -34,28 +30,30 @@ LexerModule::LexerModule(int language_,
 	LexerFunction fnLexer_,
 	const char *languageName_,
 	LexerFunction fnFolder_,
-        const char *const wordListDescriptions_[],
-	int styleBits_) :
+	const char *const wordListDescriptions_[],
+	const LexicalClass *lexClasses_,
+	size_t nClasses_) :
 	language(language_),
 	fnLexer(fnLexer_),
 	fnFolder(fnFolder_),
 	fnFactory(0),
 	wordListDescriptions(wordListDescriptions_),
-	styleBits(styleBits_),
+	lexClasses(lexClasses_),
+	nClasses(nClasses_),
 	languageName(languageName_) {
 }
 
 LexerModule::LexerModule(int language_,
 	LexerFactoryFunction fnFactory_,
 	const char *languageName_,
-	const char * const wordListDescriptions_[],
-	int styleBits_) :
+	const char * const wordListDescriptions_[]) :
 	language(language_),
 	fnLexer(0),
 	fnFolder(0),
 	fnFactory(fnFactory_),
 	wordListDescriptions(wordListDescriptions_),
-	styleBits(styleBits_),
+	lexClasses(nullptr),
+	nClasses(0),
 	languageName(languageName_) {
 }
 
@@ -79,34 +77,38 @@ const char *LexerModule::GetWordListDescription(int index) const {
 		return "";
 	} else {
 		return wordListDescriptions[index];
- 	}
+	}
 }
 
-int LexerModule::GetStyleBitsNeeded() const {
-	return styleBits;
+const LexicalClass *LexerModule::LexClasses() const {
+	return lexClasses;
 }
 
-ILexer *LexerModule::Create() const {
+size_t LexerModule::NamedStyles() const {
+	return nClasses;
+}
+
+ILexer4 *LexerModule::Create() const {
 	if (fnFactory)
 		return fnFactory();
 	else
 		return new LexerSimple(this);
 }
 
-void LexerModule::Lex(unsigned int startPos, int lengthDoc, int initStyle,
+void LexerModule::Lex(Sci_PositionU startPos, Sci_Position lengthDoc, int initStyle,
 	  WordList *keywordlists[], Accessor &styler) const {
 	if (fnLexer)
 		fnLexer(startPos, lengthDoc, initStyle, keywordlists, styler);
 }
 
-void LexerModule::Fold(unsigned int startPos, int lengthDoc, int initStyle,
+void LexerModule::Fold(Sci_PositionU startPos, Sci_Position lengthDoc, int initStyle,
 	  WordList *keywordlists[], Accessor &styler) const {
 	if (fnFolder) {
-		int lineCurrent = styler.GetLine(startPos);
+		Sci_Position lineCurrent = styler.GetLine(startPos);
 		// Move back one line in case deletion wrecked current line fold state
 		if (lineCurrent > 0) {
 			lineCurrent--;
-			int newStartPos = styler.LineStart(lineCurrent);
+			Sci_Position newStartPos = styler.LineStart(lineCurrent);
 			lengthDoc += startPos - newStartPos;
 			startPos = newStartPos;
 			initStyle = 0;
