@@ -10,6 +10,7 @@
 
 #include "PlatQt.h"
 #include "Scintilla.h"
+#include "DBCS.h"
 #include "FontQuality.h"
 
 #include <QApplication>
@@ -281,13 +282,12 @@ void SurfaceImpl::Polygon(Point *pts,
 	PenColour(fore);
 	BrushColour(back);
 
-	QPoint *qpts = new QPoint[npts];
+	std::vector<QPoint> qpts(npts);
 	for (int i = 0; i < npts; i++) {
 		qpts[i] = QPoint(pts[i].x, pts[i].y);
 	}
 
-	GetPainter()->drawPolygon(qpts, npts);
-	delete [] qpts;
+	GetPainter()->drawPolygon(&qpts[0], npts);
 }
 
 void SurfaceImpl::RectangleDraw(PRectangle rc,
@@ -490,7 +490,7 @@ void SurfaceImpl::MeasureWidths(Font &font,
 		// DBCS
 		int ui = 0;
 		for (int i=0; i<len;) {
-			size_t lenChar = Platform::IsDBCSLeadByte(codePage, s[i]) ? 2 : 1;
+			size_t lenChar = DBCSIsLeadByte(codePage, s[i]) ? 2 : 1;
 			qreal xPosition = tl.cursorToX(ui+1);
 			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<len); bytePos++) {
 				positions[i++] = xPosition;
@@ -540,12 +540,6 @@ XYPOSITION SurfaceImpl::InternalLeading(Font & /* font */)
 	return 0;
 }
 
-XYPOSITION SurfaceImpl::ExternalLeading(Font &font)
-{
-	QFontMetricsF metrics(*FontPointer(font), device);
-	return metrics.leading();
-}
-
 XYPOSITION SurfaceImpl::Height(Font &font)
 {
 	QFontMetricsF metrics(*FontPointer(font), device);
@@ -626,11 +620,6 @@ void Window::Destroy()
 	wid = 0;
 }
 
-bool Window::HasFocus()
-{
-	return wid ? window(wid)->hasFocus() : false;
-}
-
 PRectangle Window::GetPosition()
 {
 	// Before any size allocated pretend its 1000 wide so not scrolled
@@ -726,12 +715,6 @@ void Window::SetCursor(Cursor curs)
 	}
 }
 
-void Window::SetTitle(const char *s)
-{
-	if (wid)
-		window(wid)->setWindowTitle(s);
-}
-
 /* Returns rectangle of monitor pt is on, both rect and pt are in Window's
    window coordinates */
 PRectangle Window::GetMonitorRect(Point pt)
@@ -753,28 +736,28 @@ public:
 	ListBoxImpl();
 	~ListBoxImpl();
 
-	virtual void SetFont(Font &font);
-	virtual void Create(Window &parent, int ctrlID, Point location,
-						int lineHeight, bool unicodeMode, int technology);
-	virtual void SetAverageCharWidth(int width);
-	virtual void SetVisibleRows(int rows);
-	virtual int GetVisibleRows() const;
-	virtual PRectangle GetDesiredRect();
-	virtual int CaretFromEdge();
-	virtual void Clear();
-	virtual void Append(char *s, int type = -1);
-	virtual int Length();
-	virtual void Select(int n);
-	virtual int GetSelection();
-	virtual int Find(const char *prefix);
-	virtual void GetValue(int n, char *value, int len);
-	virtual void RegisterImage(int type, const char *xpmData);
-	virtual void RegisterRGBAImage(int type, int width, int height,
-		const unsigned char *pixelsImage);
+	void SetFont(Font &font) override;
+	void Create(Window &parent, int ctrlID, Point location,
+						int lineHeight, bool unicodeMode_, int technology) override;
+	void SetAverageCharWidth(int width) override;
+	void SetVisibleRows(int rows) override;
+	int GetVisibleRows() const override;
+	PRectangle GetDesiredRect() override;
+	int CaretFromEdge() override;
+	void Clear() override;
+	void Append(char *s, int type = -1) override;
+	int Length() override;
+	void Select(int n) override;
+	int GetSelection() override;
+	int Find(const char *prefix) override;
+	void GetValue(int n, char *value, int len) override;
+	void RegisterImage(int type, const char *xpmData) override;
+	void RegisterRGBAImage(int type, int width, int height,
+		const unsigned char *pixelsImage) override;
 	virtual void RegisterQPixmapImage(int type, const QPixmap& pm);
-	virtual void ClearRegisteredImages();
-	virtual void SetDoubleClickAction(CallBackAction action, void *data);
-	virtual void SetList(const char *list, char separator, char typesep);
+	void ClearRegisteredImages() override;
+	void SetDelegate(IListBoxDelegate *lbDelegate) override;
+	void SetList(const char *list, char separator, char typesep) override;
 private:
 	bool unicodeMode;
 	int visibleRows;
@@ -786,15 +769,16 @@ public:
 	explicit ListWidget(QWidget *parent);
 	virtual ~ListWidget();
 
-	void setDoubleClickAction(CallBackAction action, void *data);
+	void setDelegate(IListBoxDelegate *lbDelegate);
+	void selectionChanged();
 
 protected:
-	virtual void mouseDoubleClickEvent(QMouseEvent *event);
-	virtual QStyleOptionViewItem viewOptions() const;
+	void mouseReleaseEvent(QMouseEvent * event) override;
+	void mouseDoubleClickEvent(QMouseEvent *event) override;
+	QStyleOptionViewItem viewOptions() const override;
 
 private:
-	CallBackAction doubleClickAction;
-	void *doubleClickActionData;
+	IListBoxDelegate *delegate;
 };
 
 
@@ -819,7 +803,11 @@ void ListBoxImpl::Create(Window &parent,
 #if defined(Q_OS_WIN)
 	// On Windows, Qt::ToolTip causes a crash when the list is clicked on
 	// so Qt::Tool is used.
-	list->setParent(0, Qt::Tool | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
+	list->setParent(0, Qt::Tool | Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
+		| Qt::WindowDoesNotAcceptFocus
+#endif
+	);
 #else
 	// On OS X, Qt::Tool takes focus so main window loses focus so
 	// keyboard stops working. Qt::ToolTip works but its only really
@@ -943,6 +931,7 @@ void ListBoxImpl::Select(int n)
 		}
 	}
 	list->setCurrentRow(n);
+	list->selectionChanged();
 }
 
 int ListBoxImpl::GetSelection()
@@ -984,7 +973,7 @@ void ListBoxImpl::RegisterQPixmapImage(int type, const QPixmap& pm)
 	if (list != NULL) {
 		QSize iconSize = list->iconSize();
 		if (pm.width() > iconSize.width() || pm.height() > iconSize.height())
-			list->setIconSize(QSize(qMax(pm.width(), iconSize.width()), 
+			list->setIconSize(QSize(qMax(pm.width(), iconSize.width()),
 						 qMax(pm.height(), iconSize.height())));
 	}
 
@@ -1005,16 +994,16 @@ void ListBoxImpl::RegisterRGBAImage(int type, int width, int height, const unsig
 void ListBoxImpl::ClearRegisteredImages()
 {
 	images.clear();
-	
+
 	ListWidget *list = static_cast<ListWidget *>(wid);
 	if (list != NULL)
 		list->setIconSize(QSize(0, 0));
 }
 
-void ListBoxImpl::SetDoubleClickAction(CallBackAction action, void *data)
+void ListBoxImpl::SetDelegate(IListBoxDelegate *lbDelegate)
 {
 	ListWidget *list = static_cast<ListWidget *>(wid);
-	list->setDoubleClickAction(action, data);
+	list->setDelegate(lbDelegate);
 }
 
 void ListBoxImpl::SetList(const char *list, char separator, char typesep)
@@ -1056,24 +1045,36 @@ ListBox *ListBox::Allocate()
 }
 
 ListWidget::ListWidget(QWidget *parent)
-: QListWidget(parent), doubleClickAction(0), doubleClickActionData(0)
+: QListWidget(parent), delegate(0)
 {}
 
 ListWidget::~ListWidget() {}
 
-void ListWidget::setDoubleClickAction(CallBackAction action, void *data)
+void ListWidget::setDelegate(IListBoxDelegate *lbDelegate)
 {
-	doubleClickAction = action;
-	doubleClickActionData = data;
+	delegate = lbDelegate;
+}
+
+void ListWidget::selectionChanged() {
+	if (delegate) {
+		ListBoxEvent event(ListBoxEvent::EventType::selectionChange);
+		delegate->ListNotify(&event);
+	}
 }
 
 void ListWidget::mouseDoubleClickEvent(QMouseEvent * /* event */)
 {
-	if (doubleClickAction != 0) {
-		doubleClickAction(doubleClickActionData);
+	if (delegate) {
+		ListBoxEvent event(ListBoxEvent::EventType::doubleClick);
+		delegate->ListNotify(&event);
 	}
 }
 
+void ListWidget::mouseReleaseEvent(QMouseEvent * /* event */)
+{
+	selectionChanged();
+}
+
 QStyleOptionViewItem ListWidget::viewOptions() const
 {
 	QStyleOptionViewItem result = QListWidget::viewOptions();
@@ -1124,7 +1125,7 @@ public:
 		lib = 0;
 	}
 
-	virtual Function FindFunction(const char *name) {
+	Function FindFunction(const char *name) override {
 		if (lib) {
 			// C++ standard doesn't like casts between function pointers and void pointers so use a union
 			union {
@@ -1141,7 +1142,7 @@ public:
 		return NULL;
 	}
 
-	virtual bool IsValid() {
+	bool IsValid() override {
 		return lib != NULL;
 	}
 };
@@ -1184,47 +1185,6 @@ unsigned int Platform::DoubleClickTime()
 	return QApplication::doubleClickInterval();
 }
 
-bool Platform::MouseButtonBounce()
-{
-	return false;
-}
-
-bool Platform::IsKeyDown(int /*key*/)
-{
-	return false;
-}
-
-long Platform::SendScintilla(WindowID /*w*/,
-                             unsigned int /*msg*/,
-                             unsigned long /*wParam*/,
-                             long /*lParam*/)
-{
-	return 0;
-}
-
-long Platform::SendScintillaPointer(WindowID /*w*/,
-                                    unsigned int /*msg*/,
-                                    unsigned long /*wParam*/,
-                                    void * /*lParam*/)
-{
-	return 0;
-}
-
-int Platform::Minimum(int a, int b)
-{
-	return qMin(a, b);
-}
-
-int Platform::Maximum(int a, int b)
-{
-	return qMax(a, b);
-}
-
-int Platform::Clamp(int val, int minVal, int maxVal)
-{
-	return qBound(minVal, val, maxVal);
-}
-
 void Platform::DebugDisplay(const char *s)
 {
 	qWarning("Scintilla: %s", s);
@@ -1259,51 +1219,6 @@ void Platform::Assert(const char *c, const char *file, int line)
 	}
 }
 
-
-bool Platform::IsDBCSLeadByte(int codePage, char ch)
-{
-	// Byte ranges found in Wikipedia articles with relevant search strings in each case
-	unsigned char uch = static_cast<unsigned char>(ch);
-	switch (codePage) {
-	case 932:
-		// Shift_jis
-		return ((uch >= 0x81) && (uch <= 0x9F)) ||
-		       ((uch >= 0xE0) && (uch <= 0xEF));
-	case 936:
-		// GBK
-		return (uch >= 0x81) && (uch <= 0xFE);
-	case 949:
-		// Korean Wansung KS C-5601-1987
-		return (uch >= 0x81) && (uch <= 0xFE);
-	case 950:
-		// Big5
-		return (uch >= 0x81) && (uch <= 0xFE);
-	case 1361:
-		// Korean Johab KS C-5601-1992
-		return
-		    ((uch >= 0x84) && (uch <= 0xD3)) ||
-		    ((uch >= 0xD8) && (uch <= 0xDE)) ||
-		    ((uch >= 0xE0) && (uch <= 0xF9));
-	}
-	return false;
-}
-
-int Platform::DBCSCharLength(int codePage, const char *s)
-{
-	if (codePage == 932 || codePage == 936 || codePage == 949 ||
-	        codePage == 950 || codePage == 1361) {
-		return IsDBCSLeadByte(codePage, s[0]) ? 2 : 1;
-	} else {
-		return 1;
-	}
-}
-
-int Platform::DBCSCharMaxLength()
-{
-	return 2;
-}
-
-
 //----------------------------------------------------------------------
 
 static QElapsedTimer timer;
