@@ -38,12 +38,14 @@ ScintillaQt::ScintillaQt(QAbstractScrollArea *parent)
 
 	wMain = scrollArea->viewport();
 
+	imeInteraction = imeInline;
+
 	// On OS X drawing text into a pixmap moves it around 1 pixel to
 	// the right compared to drawing it directly onto a window.
 	// Buffered drawing turned off by default to avoid this.
 	WndProc(SCI_SETBUFFEREDDRAW, false, 0);
 
-	Initialise();
+	Init();
 
 	for (TickReason tr = tickCaret; tr <= tickDwell; tr = static_cast<TickReason>(tr + 1)) {
 		timers[tr] = 0;
@@ -126,13 +128,9 @@ static ScintillaRectangularMime *singletonMime = 0;
 
 #endif
 
-void ScintillaQt::Initialise()
+void ScintillaQt::Init()
 {
-#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
 	rectangularSelectionModifier = SCMOD_ALT;
-#else
-	rectangularSelectionModifier = SCMOD_CTRL;
-#endif
 
 #if defined(Q_OS_MAC) && QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
 	if (!singletonMime) {
@@ -166,8 +164,8 @@ void ScintillaQt::SelectionChanged()
 
 bool ScintillaQt::DragThreshold(Point ptStart, Point ptNow)
 {
-	int xMove = abs(ptStart.x - ptNow.x);
-	int yMove = abs(ptStart.y - ptNow.y);
+	int xMove = std::abs(ptStart.x - ptNow.x);
+	int yMove = std::abs(ptStart.y - ptNow.y);
 	return (xMove > QApplication::startDragDistance()) ||
 		(yMove > QApplication::startDragDistance());
 }
@@ -237,7 +235,7 @@ bool ScintillaQt::ValidCodePage(int codePage) const
 }
 
 
-void ScintillaQt::ScrollText(int linesToMove)
+void ScintillaQt::ScrollText(Sci::Line linesToMove)
 {
 	int dy = vs.lineHeight * (linesToMove);
 	scrollArea->viewport()->scroll(0, dy);
@@ -255,7 +253,7 @@ void ScintillaQt::SetHorizontalScrollPos()
 	emit horizontalScrolled(xOffset);
 }
 
-bool ScintillaQt::ModifyScrollBars(int nMax, int nPage)
+bool ScintillaQt::ModifyScrollBars(Sci::Line nMax, Sci::Line nPage)
 {
 	bool modified = false;
 
@@ -399,12 +397,13 @@ void ScintillaQt::NotifyParent(SCNotification scn)
 	emit notifyParent(scn);
 }
 
-/**
-* Report that this Editor subclass has a working implementation of FineTickerStart.
-*/
-bool ScintillaQt::FineTickerAvailable()
+void ScintillaQt::NotifyURIDropped(const char *uri)
 {
-	return true;
+	SCNotification scn = {};
+	scn.nmhdr.code = SCN_URIDROPPED;
+	scn.text = uri;
+
+	NotifyParent(scn);
 }
 
 bool ScintillaQt::FineTickerRunning(TickReason reason)
@@ -496,10 +495,10 @@ QByteArray ScintillaQt::BytesForDocument(const QString &text) const
 class CaseFolderDBCS : public CaseFolderTable {
 	QTextCodec *codec;
 public:
-	CaseFolderDBCS(QTextCodec *codec_) : codec(codec_) {
+	explicit CaseFolderDBCS(QTextCodec *codec_) : codec(codec_) {
 		StandardASCII();
 	}
-	virtual size_t Fold(char *folded, size_t sizeFolded, const char *mixed, size_t lenMixed) {
+	size_t Fold(char *folded, size_t sizeFolded, const char *mixed, size_t lenMixed) override {
 		if ((lenMixed == 1) && (sizeFolded > 0)) {
 			folded[0] = mapping[static_cast<unsigned char>(mixed[0])];
 			return 1;
@@ -611,7 +610,7 @@ void ScintillaQt::StartDrag()
 		}
 	}
 	inDragDrop = ddNone;
-	SetDragPosition(SelectionPosition(invalidPosition));
+	SetDragPosition(SelectionPosition(Sci::invalidPosition));
 }
 
 void ScintillaQt::CreateCallTipWindow(PRectangle rc)
@@ -646,10 +645,14 @@ void ScintillaQt::AddToPopUp(const char *label,
 	        this, SLOT(execCommand(QAction *)));
 }
 
-sptr_t ScintillaQt::WndProc(unsigned int message, uptr_t wParam, sptr_t lParam)
+sptr_t ScintillaQt::WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam)
 {
 	try {
-		switch (message) {
+		switch (iMessage) {
+
+		case SCI_SETIMEINTERACTION:
+			// Only inline IME supported on Qt
+			break;
 
 		case SCI_GRABFOCUS:
 			scrollArea->setFocus(Qt::OtherFocusReason);
@@ -668,7 +671,7 @@ sptr_t ScintillaQt::WndProc(unsigned int message, uptr_t wParam, sptr_t lParam)
 #endif
 
 		default:
-			return ScintillaBase::WndProc(message, wParam, lParam);
+			return ScintillaBase::WndProc(iMessage, wParam, lParam);
 		}
 	} catch (std::bad_alloc &) {
 		errorStatus = SC_STATUS_BADALLOC;
@@ -734,7 +737,7 @@ void ScintillaQt::DragMove(const Point &point)
 
 void ScintillaQt::DragLeave()
 {
-	SetDragPosition(SelectionPosition(invalidPosition));
+	SetDragPosition(SelectionPosition(Sci::invalidPosition));
 }
 
 void ScintillaQt::Drop(const Point &point, const QMimeData *data, bool move)
@@ -750,6 +753,13 @@ void ScintillaQt::Drop(const Point &point, const QMimeData *data, bool move)
 	DropAt(movePos, bytes, len, move, rectangular);
 }
 
+void ScintillaQt::DropUrls(const QMimeData *data)
+{
+	foreach(const QUrl &url, data->urls()) {
+		NotifyURIDropped(url.toString().toUtf8().constData());
+	}
+}
+
 void ScintillaQt::timerEvent(QTimerEvent *event)
 {
 	for (TickReason tr=tickCaret; tr<=tickDwell; tr = static_cast<TickReason>(tr+1)) {
