@@ -18,6 +18,7 @@
 #include <vector>
 #include <set>
 #include <map>
+#include <memory>
 #include <sstream>
 
 #include "Scintilla.h"
@@ -112,7 +113,7 @@ static std::string GetRTFStyleChange(const char *last, const char *current) { //
 }
 
 void SciTEBase::SaveToStreamRTF(std::ostream &os, int start, int end) {
-	int lengthDoc = LengthDocument();
+	const int lengthDoc = LengthDocument();
 	if (end < 0)
 		end = lengthDoc;
 	RemoveFindMarks();
@@ -121,14 +122,14 @@ void SciTEBase::SaveToStreamRTF(std::ostream &os, int start, int end) {
 	StyleDefinition defaultStyle = StyleDefinitionFor(STYLE_DEFAULT);
 
 	int tabSize = props.GetInt("export.rtf.tabsize", props.GetInt("tabsize"));
-	int wysiwyg = props.GetInt("export.rtf.wysiwyg", 1);
+	const int wysiwyg = props.GetInt("export.rtf.wysiwyg", 1);
 	std::string fontFace = props.GetExpandedString("export.rtf.font.face");
 	if (fontFace.length()) {
 		defaultStyle.font = fontFace;
 	} else if (defaultStyle.font.length() == 0) {
 		defaultStyle.font = RTF_FONTFACE;
 	}
-	int fontSize = props.GetInt("export.rtf.font.size", 0);
+	const int fontSize = props.GetInt("export.rtf.font.size", 0);
 	if (fontSize > 0) {
 		defaultStyle.size = fontSize << 1;
 	} else if (defaultStyle.size == 0) {
@@ -138,7 +139,7 @@ void SciTEBase::SaveToStreamRTF(std::ostream &os, int start, int end) {
 	}
 	const bool isUTF8 = wEditor.Call(SCI_GETCODEPAGE) == SC_CP_UTF8;
 	unsigned int characterset = props.GetInt("character.set", SC_CHARSET_DEFAULT);
-	int tabs = props.GetInt("export.rtf.tabs", 0);
+	const int tabs = props.GetInt("export.rtf.tabs", 0);
 	if (tabSize == 0)
 		tabSize = 4;
 
@@ -205,9 +206,9 @@ void SciTEBase::SaveToStreamRTF(std::ostream &os, int start, int end) {
 		styles.push_back(osStyle.str());
 	}
 	os << RTF_FONTDEFCLOSE RTF_COLORDEFOPEN;
-	for (size_t i = 0; i < colors.size(); i++) {
-		os << "\\red" << IntFromHexByte(colors[i].c_str() + 1) << "\\green" << IntFromHexByte(colors[i].c_str() + 3) <<
-		   "\\blue" << IntFromHexByte(colors[i].c_str() + 5) << ";";
+	for (const std::string &color : colors) {
+		os << "\\red" << IntFromHexByte(color.c_str() + 1) << "\\green" << IntFromHexByte(color.c_str() + 3) <<
+		   "\\blue" << IntFromHexByte(color.c_str() + 5) << ";";
 	}
 	os << RTF_COLORDEFCLOSE RTF_HEADERCLOSE RTF_BODYOPEN RTF_SETFONTFACE "0"
 	   RTF_SETFONTSIZE << defaultStyle.size << RTF_SETCOLOR "0 ";
@@ -221,7 +222,7 @@ void SciTEBase::SaveToStreamRTF(std::ostream &os, int start, int end) {
 	TextReader acc(wEditor);
 	int column = 0;
 	for (int iPos = start; iPos < end; iPos++) {
-		char ch = acc[iPos];
+		const char ch = acc[iPos];
 		int style = acc.StyleAt(iPos);
 		if (style > STYLE_MAX)
 			style = 0;
@@ -242,7 +243,7 @@ void SciTEBase::SaveToStreamRTF(std::ostream &os, int start, int end) {
 			if (tabs) {
 				os << RTF_TAB;
 			} else {
-				int ts = tabSize - (column % tabSize);
+				const int ts = tabSize - (column % tabSize);
 				for (int itab = 0; itab < ts; itab++) {
 					os << ' ';
 				}
@@ -282,13 +283,17 @@ void SciTEBase::SaveToRTF(const FilePath &saveName, int start, int end) {
 	FILE *fp = saveName.Open(GUI_TEXT("wt"));
 	bool failedWrite = fp == NULL;
 	if (fp) {
-		std::ostringstream oss;
-		SaveToStreamRTF(oss, start, end);
-		std::string rtf = oss.str();
-		if (fwrite(rtf.c_str(), 1, rtf.length(), fp) != rtf.length()) {
-			failedWrite = true;
-		}
-		if (fclose(fp) != 0) {
+		try {
+			std::ostringstream oss;
+			SaveToStreamRTF(oss, start, end);
+			std::string rtf = oss.str();
+			if (fwrite(rtf.c_str(), 1, rtf.length(), fp) != rtf.length()) {
+				failedWrite = true;
+			}
+			if (fclose(fp) != 0) {
+				failedWrite = true;
+			}
+		} catch (std::exception &) {
 			failedWrite = true;
 		}
 	}
