@@ -133,7 +133,7 @@ struct ScintillaFailure {
 };
 
 class ScintillaWindow : public Window {
-	// Deleted so ScintillaWindow objects can not be copied
+	// Deleted so ScintillaWindow objects can not be copied.
 	ScintillaWindow(const ScintillaWindow &source) = delete;
 	ScintillaWindow &operator=(const ScintillaWindow &) = delete;
 	SciFnDirect fn;
@@ -168,7 +168,7 @@ public:
 		}
 		if (!fn)
 			throw ScintillaFailure(SC_STATUS_FAILURE);
-		sptr_t retVal = fn(ptr, msg, wParam, lParam);
+		const sptr_t retVal = fn(ptr, msg, wParam, lParam);
 		status = fn(ptr, SCI_GETSTATUS, 0, 0);
 		if (status > 0 && status < SC_STATUS_WARN_START)
 			throw ScintillaFailure(status);
