@@ -40,6 +40,9 @@
 class MultiplexExtension: public Extension {
 public:
 	MultiplexExtension();
+	// Copying is unsupported.
+	MultiplexExtension(const MultiplexExtension & copy) = delete;
+	MultiplexExtension & operator=(const MultiplexExtension & copy) = delete;
 	virtual ~MultiplexExtension();
 
 	bool RegisterExtension(Extension &ext_);
@@ -77,13 +80,8 @@ public:
 	virtual bool NeedsOnClose();
 
 private:
-	Extension **extensions;
-	int extensionCount;
+	std::vector<Extension *> extensions;
 	ExtensionAPI *host;
-
-	// Copying is unsupported.
-	MultiplexExtension(const MultiplexExtension & copy);
-	MultiplexExtension & operator=(const MultiplexExtension & copy);
 };
 
 #endif
