@@ -30,8 +30,8 @@ public:
 	PropSetFile *superPS;
 	explicit PropSetFile(bool lowerKeys_=false);
 	PropSetFile(const PropSetFile &copy);
-	virtual ~PropSetFile();
 	PropSetFile &operator=(const PropSetFile &assign);
+	virtual ~PropSetFile();
 	void Set(const char *key, const char *val, ptrdiff_t lenKey=-1, ptrdiff_t lenVal=-1);
 	void Set(const char *keyVal);
 	void Unset(const char *key, int lenKey=-1);
@@ -45,7 +45,7 @@ public:
 	void Clear();
 
 	enum ReadLineState { rlActive, rlExcludedModule, rlConditionFalse };
-	ReadLineState ReadLine(const char *data, ReadLineState rls, const FilePath &directoryForImports, const ImportFilter &filter,
+	ReadLineState ReadLine(const char *lineBuffer, ReadLineState rls, const FilePath &directoryForImports, const ImportFilter &filter,
 		FilePathSet *imports, size_t depth);
 	void ReadFromMemory(const char *data, size_t len, const FilePath &directoryForImports, const ImportFilter &filter,
 		FilePathSet *imports, size_t depth);
