@@ -168,7 +168,7 @@ public:
 	}
 	void Allocate(int maxSize);
 	int Add();
-	int GetDocumentByWorker(FileWorker *pFileWorker) const;
+	int GetDocumentByWorker(const FileWorker *pFileWorker) const;
 	int GetDocumentByName(const FilePath &filename, bool excludeCurrent=false);
 	void RemoveInvisible(int index);
 	void RemoveCurrent();
@@ -406,6 +406,7 @@ protected:
 	GUI::Window wContent;    ///< Contains wEditor and wOutput
 	GUI::ScintillaWindow wEditor;
 	GUI::ScintillaWindow wOutput;
+	GUI::ScintillaWindow *pwFocussed;
 	GUI::Window wIncrement;
 	GUI::Window wToolBar;
 	GUI::Window wStatusBar;
@@ -432,6 +433,8 @@ protected:
 	int wrapStyle;
 	int alphaIndicator;
 	bool underIndicator;
+	std::string foldColour;
+	std::string foldHiliteColour;
 	bool openFilesHere;
 	bool fullScreen;
 	enum { toolMax = 50 };
@@ -526,7 +529,6 @@ protected:
 	// Handle buffers
 	sptr_t GetDocumentAt(int index);
 	void SwitchDocumentAt(int index, sptr_t pdoc);
-	int AddBuffer();
 	void UpdateBuffersCurrent();
 	bool IsBufferAvailable() const;
 	bool CanMakeRoom(bool maySaveIfDirty = true);
@@ -559,6 +561,7 @@ protected:
 	void ReadLocalPropFile();
 	void ReadDirectoryPropFile();
 
+	void SetPaneFocus(bool editPane);
 	int CallFocused(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	int CallFocusedElseDefault(int defaultValue, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	sptr_t CallPane(int destination, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
@@ -596,7 +599,6 @@ protected:
 	void New();
 	void RestoreState(const Buffer &buffer, bool restoreBookmarks);
 	void Close(bool updateUI = true, bool loadingSession = false, bool makingRoomForNew = false);
-	bool IsAbsolutePath(const char *path);
 	static bool Exists(const GUI::gui_char *dir, const GUI::gui_char *path, FilePath *resultPath);
 	void DiscoverEOLSetting();
 	void DiscoverIndentSetting();
@@ -616,8 +618,8 @@ protected:
 	    ofQuiet = 8,		// Avoid "Could not open file" message
 	    ofSynchronous = 16	// Force synchronous read
 	};
-	void TextRead(FileWorker *pFileLoader);
-	void TextWritten(FileWorker *pFileStorer);
+	void TextRead(FileWorker *pFileWorker);
+	void TextWritten(FileWorker *pFileWorker);
 	void UpdateProgress(Worker *pWorker);
 	void PerformDeferredTasks();
 	enum OpenCompletion { ocSynchronous, ocCompleteCurrent, ocCompleteSwitch };
@@ -671,10 +673,10 @@ protected:
 	static int GetMenuCommandAsInt(std::string commandName);
 	virtual void Print(bool) {}
 	virtual void PrintSetup() {}
-	virtual void UserStripShow(const char * /* description */) {}
-	virtual void UserStripSet(int /* control */, const char * /* value */) {}
-	virtual void UserStripSetList(int /* control */, const char * /* value */) {}
-	virtual const char *UserStripValue(int /* control */) { return 0; }
+	void UserStripShow(const char * /* description */) override {}
+	void UserStripSet(int /* control */, const char * /* value */) override {}
+	void UserStripSetList(int /* control */, const char * /* value */) override {}
+	const char *UserStripValue(int /* control */) override { return 0; }
 	virtual void ShowBackgroundProgress(const GUI::gui_string & /* explanation */, size_t /* size */, size_t /* progress */) {}
 	Sci_CharacterRange GetSelection();
 	SelectedRange GetSelectedRange();
@@ -716,25 +718,26 @@ protected:
 	virtual void FindMessageBox(const std::string &msg, const std::string *findItem = 0) = 0;
 	bool FindReplaceAdvanced() const;
 	int FindInTarget(std::string findWhatText, int startPosition, int endPosition);
-	virtual void SetFindText(const char *sFind);
-	virtual void SetFind(const char *sFind);
-	virtual bool FindHasText() const;
-	virtual void SetReplace(const char *sReplace);
-	virtual void SetCaretAsStart();
-	virtual void MoveBack();
-	virtual void ScrollEditorIfNeeded();
-	virtual int FindNext(bool reverseDirection, bool showWarnings=true, bool allowRegExp=true);
-	virtual void HideMatch();
+	// Implement Searcher
+	void SetFindText(const char *sFind) override;
+	void SetFind(const char *sFind) override;
+	bool FindHasText() const override;
+	void SetReplace(const char *sReplace) override;
+	void SetCaretAsStart() override;
+	void MoveBack() override;
+	void ScrollEditorIfNeeded() override;
+	int FindNext(bool reverseDirection, bool showWarnings=true, bool allowRegExp=true) override;
+	virtual void HideMatch() override;
 	virtual void FindIncrement() = 0;
 	int IncrementSearchMode();
 	virtual void FindInFiles() = 0;
 	virtual void Replace() = 0;
-	void ReplaceOnce(bool showWarnings=true);
+	void ReplaceOnce(bool showWarnings=true) override;
 	int DoReplaceAll(bool inSelection); // returns number of replacements or negative value if error
-	int ReplaceAll(bool inSelection);
+	int ReplaceAll(bool inSelection) override;
 	int ReplaceInBuffers();
-	virtual void UIClosed();
-	virtual void UIHasFocus();
+	void UIClosed() override;
+	void UIHasFocus() override;
 	virtual void DestroyFindReplace() = 0;
 	virtual void GoLineDialog() = 0;
 	virtual bool AbbrevDialog() = 0;
@@ -777,11 +780,11 @@ protected:
 	int IndentOfBlock(int line);
 	void MaintainIndentation(char ch);
 	void AutomaticIndentation(char ch);
-	void CharAdded(int ch);
+	void CharAdded(int utf32);
 	void CharAddedOutput(int ch);
 	void SetTextProperties(PropSetFile &ps);
 	virtual void SetFileProperties(PropSetFile &ps) = 0;
-	virtual void UpdateStatusBar(bool bUpdateSlowData);
+	void UpdateStatusBar(bool bUpdateSlowData) override;
 	int GetLineLength(int line);
 	int GetCurrentLineNumber();
 	int GetCurrentScrollPosition();
@@ -799,7 +802,6 @@ protected:
 	void SetLineNumberWidth();
 	void MenuCommand(int cmdID, int source = 0);
 	void FoldChanged(int line, int levelNow, int levelPrev);
-	void FoldChanged(int position);
 	void ExpandFolds(int line, bool expand, int level);
 	void FoldAll();
 	void ToggleFoldRecursive(int line, int level);
@@ -809,7 +811,7 @@ protected:
 	bool MarginClick(int position, int modifiers);
 	void NewLineInOutput();
 	virtual void SetStatusBarText(const char *s) = 0;
-	virtual void Notify(const SCNotification *notification);
+	virtual void Notify(SCNotification *notification);
 	virtual void ShowToolBar() = 0;
 	virtual void ShowTabBar() = 0;
 	virtual void ShowStatusBar() = 0;
@@ -817,12 +819,13 @@ protected:
 
 	void RemoveFindMarks();
 	int SearchFlags(bool regularExpressions) const;
-	void MarkAll(MarkPurpose purpose=markWithBookMarks);
+	void MarkAll(MarkPurpose purpose=markWithBookMarks) override;
 	void BookmarkAdd(int lineno = -1);
 	void BookmarkDelete(int lineno = -1);
 	bool BookmarkPresent(int lineno = -1);
 	void BookmarkToggle(int lineno = -1);
 	void BookmarkNext(bool forwardScan = true, bool select = false);
+	void BookmarkSelectAll();
 	void SetOutputVisibility(bool show);
 	virtual void ShowOutputOnMainThread();
 	void ToggleOutputVisible();
@@ -912,7 +915,7 @@ protected:
 	void ExecuteMacroCommand(const char *command);
 	void AskMacroList();
 	bool StartMacroList(const char *words);
-	void ContinueMacroList(const char *stxt);
+	void ContinueMacroList(const char *stext);
 	bool ProcessCommandLine(GUI::gui_string &args, int phase);
 	virtual bool IsStdinBlocked();
 	void OpenFromStdin(bool UseOutputPane);
@@ -923,26 +926,26 @@ protected:
 	};
 	virtual bool GrepIntoDirectory(const FilePath &directory);
 	void GrepRecursive(GrepFlags gf, const FilePath &baseDir, const char *searchString, const GUI::gui_char *fileTypes);
-	void InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const GUI::gui_char *files,
+	void InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const GUI::gui_char *fileTypes,
 			  const char *search, sptr_t &originalEnd);
-	void EnumProperties(const char *action);
+	void EnumProperties(const char *propkind);
 	void SendOneProperty(const char *kind, const char *key, const char *val);
 	void PropertyFromDirector(const char *arg);
 	void PropertyToDirector(const char *arg);
 
 	// ExtensionAPI
-	sptr_t Send(Pane p, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
-	char *Range(Pane p, int start, int end);
-	void Remove(Pane p, int start, int end);
-	void Insert(Pane p, int pos, const char *s);
-	void Trace(const char *s);
-	std::string Property(const char *key);
-	void SetProperty(const char *key, const char *val);
-	void UnsetProperty(const char *key);
-	uptr_t GetInstance();
-	void ShutDown();
-	void Perform(const char *actions);
-	void DoMenuCommand(int cmdID);
+	sptr_t Send(Pane p, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0) override;
+	char *Range(Pane p, int start, int end) override;
+	void Remove(Pane p, int start, int end) override;
+	void Insert(Pane p, int pos, const char *s) override;
+	void Trace(const char *s) override;
+	std::string Property(const char *key) override;
+	void SetProperty(const char *key, const char *val) override;
+	void UnsetProperty(const char *key) override;
+	uptr_t GetInstance() override;
+	void ShutDown() override;
+	void Perform(const char *actionList) override;
+	void DoMenuCommand(int cmdID) override;
 
 	// Valid CurrentWord characters
 	bool iswordcharforsel(char ch);
@@ -958,23 +961,23 @@ public:
 	enum { maxParam = 4 };
 
 	explicit SciTEBase(Extension *ext = 0);
+	// Deleted copy-constructor and assignment operator.
+	explicit SciTEBase(const SciTEBase&) = delete;
+	void operator=(const SciTEBase&) = delete;
 	virtual ~SciTEBase();
 
-	void ProcessExecute();
+	void Finalise();
+
 	GUI::WindowID GetID() const { return wSciTE.GetID(); }
 
 	virtual bool PerformOnNewThread(Worker *pWorker) = 0;
-	virtual void PostOnMainThread(int cmd, Worker *pWorker) = 0;
+	// WorkerListener
+	virtual void PostOnMainThread(int cmd, Worker *pWorker) override = 0;
 	virtual void WorkerCommand(int cmd, Worker *pWorker);
-
-private:
-	// un-implemented copy-constructor and assignment operator
-	explicit SciTEBase(const SciTEBase&);
-	void operator=(const SciTEBase&);
 };
 
 int ControlIDOfCommand(unsigned long);
-long ColourOfProperty(PropSetFile &props, const char *key, Colour colourDefault);
+long ColourOfProperty(const PropSetFile &props, const char *key, Colour colourDefault);
 void WindowSetFocus(GUI::ScintillaWindow &w);
 
 inline bool isspacechar(unsigned char ch) {
