@@ -18,6 +18,7 @@
 #include <set>
 #include <map>
 #include <algorithm>
+#include <memory>
 
 #include "Scintilla.h"
 #include "SciLexer.h"
@@ -48,7 +49,7 @@ void Buffer::DocumentModified() {
 }
 
 bool Buffer::NeedsSave(int delayBeforeSave) const {
-	time_t now = time(0);
+	const time_t now = time(0);
 	return now && documentModTime && isDirty && !pFileWorker && (now-documentModTime > delayBeforeSave) && !IsUntitled() && !failedSave;
 }
 
@@ -70,7 +71,7 @@ void Buffer::CompleteStoring() {
 
 void Buffer::AbandonAutomaticSave() {
 	if (pFileWorker && !pFileWorker->IsLoading()) {
-		FileStorer *pFileStorer = static_cast<FileStorer *>(pFileWorker);
+		const FileStorer *pFileStorer = static_cast<FileStorer *>(pFileWorker);
 		if (!pFileStorer->visibleProgress) {
 			pFileWorker->Cancel();
 			// File is in partially saved state so may be better to remove
@@ -113,7 +114,7 @@ int BufferList::Add() {
 	return lengthVisible - 1;
 }
 
-int BufferList::GetDocumentByWorker(FileWorker *pFileWorker) const {
+int BufferList::GetDocumentByWorker(const FileWorker *pFileWorker) const {
 	for (int i = 0;i < length;i++) {
 		if (buffers[i].pFileWorker == pFileWorker) {
 			return i;
@@ -164,10 +165,11 @@ void BufferList::RemoveCurrent() {
 
 		buffers[length].Init();
 		if (current >= lengthVisible) {
-			SetCurrent(lengthVisible - 1);
-		}
-		if (current < 0) {
-			SetCurrent(0);
+			if (lengthVisible > 0) {
+				SetCurrent(lengthVisible - 1);
+			} else {
+				SetCurrent(0);
+			}
 		}
 	} else {
 		buffers[current].Init();
@@ -288,7 +290,7 @@ BackgroundActivities BufferList::CountBackgroundActivities() const {
 		if (buffers[i].pFileWorker) {
 			if (!buffers[i].pFileWorker->FinishedJob()) {
 				if (!buffers[i].pFileWorker->IsLoading()) {
-					FileStorer *fstorer = static_cast<FileStorer*>(buffers[i].pFileWorker);
+					const FileStorer *fstorer = static_cast<FileStorer*>(buffers[i].pFileWorker);
 					if (!fstorer->visibleProgress)
 						continue;
 				}
@@ -376,7 +378,7 @@ void SciTEBase::SwitchDocumentAt(int index, sptr_t pdoc) {
 }
 
 void SciTEBase::SetDocumentAt(int index, bool updateStack) {
-	int currentbuf = buffers.Current();
+	const int currentbuf = buffers.Current();
 
 	if (	index < 0 ||
 	        index >= buffers.length ||
@@ -452,7 +454,7 @@ void SciTEBase::UpdateBuffersCurrent() {
 
 			if (props.GetInt("fold")) {
 				for (int line = 0; ; line++) {
-					int lineNext = wEditor.Call(SCI_CONTRACTEDFOLDNEXT, line);
+					const int lineNext = wEditor.Call(SCI_CONTRACTEDFOLDNEXT, line);
 					if ((line < 0) || (lineNext < line))
 						break;
 					line = lineNext;
@@ -575,7 +577,11 @@ void SciTEBase::RestoreRecentMenu() {
 	}
 }
 
-static std::vector<int> LinesFromString(const std::string &s) {
+namespace {
+
+// Line numbers are 0-based inside SciTE but are saved in session files as 1-based.
+
+std::vector<int> LinesFromString(const std::string &s) {
 	std::vector<int> result;
 	if (s.length()) {
 		size_t start = 0;
@@ -591,9 +597,23 @@ static std::vector<int> LinesFromString(const std::string &s) {
 	return result;
 }
 
+std::string StringFromLines(const std::vector<int> &lines) {
+	std::string result;
+	for (const int line : lines) {
+		if (result.length()) {
+			result.append(",");
+		}
+		std::string sLine = StdStringFromInteger(line + 1);
+		result.append(sLine);
+	}
+	return result;
+}
+
+}
+
 void SciTEBase::RestoreFromSession(const Session &session) {
-	for (std::vector<BufferState>::const_iterator bs=session.buffers.begin(); bs != session.buffers.end(); ++bs)
-		AddFileToBuffer(*bs);
+	for (const BufferState &buffer : session.buffers)
+		AddFileToBuffer(buffer);
 	int iBuffer = buffers.GetDocumentByName(session.pathActive);
 	if (iBuffer >= 0)
 		SetDocumentAt(iBuffer);
@@ -637,11 +657,11 @@ void SciTEBase::RestoreSession() {
 			session.pathActive = bufferState;
 
 		propKey = IndexPropKey("buffer", i, "scroll");
-		int scroll = propsSession.GetInt(propKey.c_str());
+		const int scroll = propsSession.GetInt(propKey.c_str());
 		bufferState.scrollPosition = scroll;
 
 		propKey = IndexPropKey("buffer", i, "position");
-		int pos = propsSession.GetInt(propKey.c_str());
+		const int pos = propsSession.GetInt(propKey.c_str());
 
 		bufferState.selection.anchor = pos - 1;
 		bufferState.selection.position = bufferState.selection.anchor;
@@ -735,18 +755,18 @@ void SciTEBase::SaveSessionFile(const GUI::gui_char *sessionName) {
 	}
 
 	if (props.GetInt("buffers") && (!defaultSession || props.GetInt("save.session"))) {
-		int curr = buffers.Current();
+		const int curr = buffers.Current();
 		for (int i = 0; i < buffers.lengthVisible; i++) {
 			if (buffers.buffers[i].IsSet() && !buffers.buffers[i].IsUntitled()) {
 				Buffer &buff = buffers.buffers[i];
 				std::string propKey = IndexPropKey("buffer", i, "path");
 				fprintf(sessionFile, "\n%s=%s\n", propKey.c_str(), buff.AsUTF8().c_str());
 
-				int pos = buff.selection.position + 1;
+				const int pos = buff.selection.position + 1;
 				propKey = IndexPropKey("buffer", i, "position");
 				fprintf(sessionFile, "%s=%d\n", propKey.c_str(), pos);
 
-				int scroll = buff.scrollPosition;
+				const int scroll = buff.scrollPosition;
 				propKey = IndexPropKey("buffer", i, "scroll");
 				fprintf(sessionFile, "%s=%d\n", propKey.c_str(), scroll);
 
@@ -756,35 +776,19 @@ void SciTEBase::SaveSessionFile(const GUI::gui_char *sessionName) {
 				}
 
 				if (props.GetInt("session.bookmarks")) {
-					bool found = false;
-					for (std::vector<int>::iterator itBM=buff.bookmarks.begin();
-						itBM != buff.bookmarks.end(); ++itBM) {
-						if (!found) {
-							propKey = IndexPropKey("buffer", i, "bookmarks");
-							fprintf(sessionFile, "%s=%d", propKey.c_str(), *itBM + 1);
-							found = true;
-						} else {
-							fprintf(sessionFile, ",%d", *itBM + 1);
-						}
+					const std::string bmString = StringFromLines(buff.bookmarks);
+					if (bmString.length()) {
+						propKey = IndexPropKey("buffer", i, "bookmarks");
+						fprintf(sessionFile, "%s=%s\n", propKey.c_str(), bmString.c_str());
 					}
-					if (found)
-						fprintf(sessionFile, "\n");
 				}
 
 				if (props.GetInt("fold") && props.GetInt("session.folds")) {
-					bool found = false;
-					for (std::vector<int>::iterator itF=buff.foldState.begin();
-						itF != buff.foldState.end(); ++itF) {
-						if (!found) {
-							propKey = IndexPropKey("buffer", i, "folds");
-							fprintf(sessionFile, "%s=%d", propKey.c_str(), *itF + 1);
-							found = true;
-						} else {
-							fprintf(sessionFile, ",%d", *itF + 1);
-						}
+					const std::string foldsString = StringFromLines(buff.foldState);
+					if (foldsString.length()) {
+						propKey = IndexPropKey("buffer", i, "folds");
+						fprintf(sessionFile, "%s=%s\n", propKey.c_str(), foldsString.c_str());
 					}
-					if (found)
-						fprintf(sessionFile, "\n");
 				}
 			}
 		}
@@ -898,13 +902,13 @@ void SciTEBase::RestoreState(const Buffer &buffer, bool restoreBookmarks) {
 	// check to see whether there is saved fold state, restore
 	if (!buffer.foldState.empty()) {
 		wEditor.Call(SCI_COLOURISE, 0, -1);
-		for (std::vector<int>::const_iterator fold=buffer.foldState.begin(); fold != buffer.foldState.end(); ++fold) {
-			wEditor.Call(SCI_TOGGLEFOLD, *fold);
+		for (const int fold : buffer.foldState) {
+			wEditor.Call(SCI_TOGGLEFOLD, fold);
 		}
 	}
 	if (restoreBookmarks) {
-		for (std::vector<int>::const_iterator mark=buffer.bookmarks.begin(); mark != buffer.bookmarks.end(); ++mark) {
-			wEditor.Call(SCI_MARKERADD, *mark, markerBookmark);
+		for (const int bookmark : buffer.bookmarks) {
+			wEditor.Call(SCI_MARKERADD, bookmark, markerBookmark);
 		}
 	}
 }
@@ -1149,11 +1153,13 @@ void SciTEBase::SetBuffersMenu() {
 				GUI::gui_string sPos = GUI::StringFromInteger((pos + 1) % 10);
 				GUI::gui_string sHotKey = GUI_TEXT("&") + sPos + GUI_TEXT(" ");
 				entry = sHotKey;	// hotkey 1..0
+				if (props.GetInt("tabbar.hide.index") == 0) {
 #if defined(_WIN32)
-				titleTab = sHotKey; // add hotkey to the tabbar
+					titleTab = sHotKey; // add hotkey to the tabbar
 #elif defined(GTK)
-				titleTab = sPos + GUI_TEXT(" ");
+					titleTab = sPos + GUI_TEXT(" ");
 #endif
+				}
 			}
 #endif
 
@@ -1319,8 +1325,10 @@ void SciTEBase::DisplayAround(const RecentFile &rf) {
 }
 
 // Next and Prev file comments.
+// StackMenuNext and StackMenuPrev only used in single buffer mode.
 // Decided that "Prev" file should mean the file you had opened last
 // This means "Next" file means the file you opened longest ago.
+
 void SciTEBase::StackMenuNext() {
 	DeleteFileStackMenu();
 	for (int stackPos = fileStackMax - 1; stackPos >= 0;stackPos--) {
@@ -1338,8 +1346,8 @@ void SciTEBase::StackMenuPrev() {
 		// May need to restore last entry if removed by StackMenu
 		RecentFile rfLast = recentFileStack[fileStackMax - 1];
 		StackMenu(0);	// Swap current with top of stack
-		for (int checkPos = 0; checkPos < fileStackMax; checkPos++) {
-			if (rfLast.SameNameAs(recentFileStack[checkPos])) {
+		for (const RecentFile &rf : recentFileStack) {
+			if (rfLast.SameNameAs(rf)) {
 				rfLast.Init();
 			}
 		}
@@ -1351,13 +1359,13 @@ void SciTEBase::StackMenuPrev() {
 		}
 		recentFileStack[fileStackMax - 1].Init();
 		// Copy current file into first empty
-		for (int emptyPos = 0; emptyPos < fileStackMax; emptyPos++) {
-			if (!recentFileStack[emptyPos].IsSet()) {
+		for (RecentFile &rf : recentFileStack) {
+			if (!rf.IsSet()) {
 				if (rfLast.IsSet()) {
-					recentFileStack[emptyPos] = rfLast;
+					rf = rfLast;
 					rfLast.Init();
 				} else {
-					recentFileStack[emptyPos] = rfCurrent;
+					rf = rfCurrent;
 					break;
 				}
 			}
@@ -1515,7 +1523,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 					while (*endPath && !isdigitchar(*endPath)) {
 						endPath++;
 					}
-					int sourceNumber = atoi(endPath) - 1;
+					const int sourceNumber = atoi(endPath) - 1;
 					return sourceNumber;
 				}
 			}
@@ -1636,8 +1644,8 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			// Lua 4 error looks like: last token read: `result' at line 40 in file `Test.lua'
 			const char *idLine = "at line ";
 			const char *idFile = "file ";
-			size_t lenLine = strlen(idLine);
-			size_t lenFile = strlen(idFile);
+			const size_t lenLine = strlen(idLine);
+			const size_t lenFile = strlen(idFile);
 			const char *line = strstr(cdoc, idLine);
 			const char *file = strstr(cdoc, idFile);
 			if (line && file) {
@@ -1680,8 +1688,8 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			// PHP error look like: Fatal error: Call to undefined function:  foo() in example.php on line 11
 			const char *idLine = " on line ";
 			const char *idFile = " in ";
-			size_t lenLine = strlen(idLine);
-			size_t lenFile = strlen(idFile);
+			const size_t lenLine = strlen(idLine);
+			const size_t lenFile = strlen(idFile);
 			const char *line = strstr(cdoc, idLine);
 			const char *file = strstr(cdoc, idFile);
 			if (line && file && (line > file)) {
@@ -1746,8 +1754,8 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			// Absoft Pro Fortran 90/95 v8.x, v9.x  errors look like: cf90-113 f90fe: ERROR SHF3D, File = shf.f90, Line = 1101, Column = 19
 			const char *idFile = " File = ";
 			const char *idLine = ", Line = ";
-			size_t lenFile = strlen(idFile);
-			size_t lenLine = strlen(idLine);
+			const size_t lenFile = strlen(idFile);
+			const size_t lenLine = strlen(idLine);
 			const char *file = strstr(cdoc, idFile);
 			const char *line = strstr(cdoc, idLine);
 			//const char *idColumn = ", Column = ";
@@ -1768,8 +1776,8 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 				 */
 			const char *idFile = ": Error: ";
 			const char *idLine = ", line ";
-			size_t lenFile = strlen(idFile);
-			size_t lenLine = strlen(idLine);
+			const size_t lenFile = strlen(idFile);
+			const size_t lenLine = strlen(idLine);
 			const char *file = strstr(cdoc, idFile);
 			const char *line = strstr(cdoc, idLine);
 			const char *lineend = strrchr(cdoc, ':');
@@ -1795,7 +1803,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 				const char *col = strchr(line + 1, ' ');
 				if (col) {
 					//*col = '\0';
-					int lnr = atoi(line) - 1;
+					const int lnr = atoi(line) - 1;
 					col = strchr(col + 1, ' ');
 					if (col) {
 						const char *endcol = strchr(col + 1, ' ');
@@ -1819,7 +1827,7 @@ static int DecodeMessage(const char *cdoc, std::string &sourcePath, int format,
 			ptrdiff_t length = endPath - startPath;
 			if (length > 0) {
 				sourcePath.assign(startPath, length);
-				int sourceNumber = atoi(endPath + 1) - 1;
+				const int sourceNumber = atoi(endPath + 1) - 1;
 				return sourceNumber;
 			}
 			break;
@@ -1872,10 +1880,10 @@ void SciTEBase::ShowMessages(int line) {
 	TextReader acc(wOutput);
 	while ((line > 0) && (acc.StyleAt(acc.LineStart(line-1)) != SCE_ERR_CMD))
 		line--;
-	int maxLine = wOutput.Call(SCI_GETLINECOUNT);
+	const int maxLine = wOutput.Call(SCI_GETLINECOUNT);
 	while ((line < maxLine) && (acc.StyleAt(acc.LineStart(line)) != SCE_ERR_CMD)) {
 		int startPosLine = wOutput.Call(SCI_POSITIONFROMLINE, line, 0);
-		int lineEnd = wOutput.Call(SCI_GETLINEENDPOSITION, line, 0);
+		const int lineEnd = wOutput.Call(SCI_GETLINEENDPOSITION, line, 0);
 		std::string message = GetRangeString(wOutput, startPosLine, lineEnd);
 		std::string source;
 		int column;
@@ -1936,8 +1944,8 @@ void SciTEBase::GoMessage(int dir) {
 	crange.cpMin = wOutput.Call(SCI_GETSELECTIONSTART);
 	crange.cpMax = wOutput.Call(SCI_GETSELECTIONEND);
 	long selStart = static_cast<long>(crange.cpMin);
-	int curLine = wOutput.Call(SCI_LINEFROMPOSITION, selStart);
-	int maxLine = wOutput.Call(SCI_GETLINECOUNT);
+	const int curLine = wOutput.Call(SCI_LINEFROMPOSITION, selStart);
+	const int maxLine = wOutput.Call(SCI_GETLINECOUNT);
 	int lookLine = curLine + dir;
 	if (lookLine < 0)
 		lookLine = maxLine - 1;
@@ -1946,7 +1954,7 @@ void SciTEBase::GoMessage(int dir) {
 	TextReader acc(wOutput);
 	while ((dir == 0) || (lookLine != curLine)) {
 		int startPosLine = wOutput.Call(SCI_POSITIONFROMLINE, lookLine, 0);
-		int lineLength = wOutput.Call(SCI_LINELENGTH, lookLine, 0);
+		const int lineLength = wOutput.Call(SCI_LINELENGTH, lookLine, 0);
 		int style = acc.StyleAt(startPosLine);
 		if (style != SCE_ERR_DEFAULT &&
 		        style != SCE_ERR_CMD &&
