@@ -21,6 +21,7 @@
 #include <set>
 #include <map>
 #include <algorithm>
+#include <memory>
 
 #include "Scintilla.h"
 #include "ILexer.h"
@@ -108,12 +109,12 @@ void SciTEBase::CountLineEnds(int &linesCR, int &linesLF, int &linesCRLF) {
 	linesCR = 0;
 	linesLF = 0;
 	linesCRLF = 0;
-	int lengthDoc = LengthDocument();
+	const int lengthDoc = LengthDocument();
 	char chPrev = ' ';
 	TextReader acc(wEditor);
 	char chNext = acc.SafeGetCharAt(0);
 	for (int i = 0; i < lengthDoc; i++) {
-		char ch = chNext;
+		const char ch = chNext;
 		chNext = acc.SafeGetCharAt(i + 1);
 		if (ch == '\r') {
 			if (chNext == '\n')
@@ -188,7 +189,7 @@ std::string SciTEBase::DiscoverLanguage() {
 }
 
 void SciTEBase::DiscoverIndentSetting() {
-	int lengthDoc = LengthDocument();
+	const int lengthDoc = std::min(LengthDocument(), 1000000);
 	TextReader acc(wEditor);
 	bool newline = true;
 	int indent = 0; // current line indentation
@@ -196,7 +197,7 @@ void SciTEBase::DiscoverIndentSetting() {
 	int prevIndent = 0; // previous line indentation
 	int prevTabSize = -1; // previous line tab size
 	for (int i = 0; i < lengthDoc; i++) {
-		char ch = acc[i];
+		const char ch = acc[i];
 		if (ch == '\r' || ch == '\n') {
 			indent = 0;
 			newline = true;
@@ -289,7 +290,7 @@ void SciTEBase::OpenCurrentFile(long long fileSize, bool suppressMessage, bool a
 		Utf8_16_Read convert;
 		std::vector<char> data(blockSize);
 		size_t lenFile = fread(&data[0], 1, data.size(), fp);
-		UniMode umCodingCookie = CodingCookieValue(&data[0], lenFile);
+		const UniMode umCodingCookie = CodingCookieValue(&data[0], lenFile);
 		while (lenFile > 0) {
 			lenFile = convert.convert(&data[0], lenFile);
 			char *dataBlock = convert.getNewBuf();
@@ -411,8 +412,8 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 	int iBuffer = buffers.GetDocumentByWorker(pFileStorer);
 
 	FilePath pathSaved = pFileStorer->path;
-	int errSaved = pFileStorer->err;
-	bool cancelledSaved = pFileStorer->Cancelling();
+	const int errSaved = pFileStorer->err;
+	const bool cancelledSaved = pFileStorer->Cancelling();
 
 	// May not be found if save cancelled or buffer closed
 	if (iBuffer >= 0) {
@@ -469,7 +470,7 @@ void SciTEBase::TextWritten(FileWorker *pFileWorker) {
 void SciTEBase::UpdateProgress(Worker *) {
 	GUI::gui_string prog;
 	BackgroundActivities bgActivities = buffers.CountBackgroundActivities();
-	int countBoth = bgActivities.loaders + bgActivities.storers;
+	const int countBoth = bgActivities.loaders + bgActivities.storers;
 	if (countBoth == 0) {
 		// Should hide UI
 		ShowBackgroundProgress(GUI_TEXT(""), 0, 0);
@@ -511,7 +512,8 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 		RemoveFileFromStack(absPath);
 		DeleteFileStackMenu();
 		SetFileStackMenu();
-		if (!(of & ofForceLoad)) // Just rotate into view
+		// If not forcing reload or currently busy with load or save, just rotate into view
+		if ((!(of & ofForceLoad)) || (CurrentBufferConst()->pFileWorker))
 			return true;
 	}
 	// See if we can have a buffer for the file to open
@@ -530,7 +532,7 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 	}
 	if (fileSize > 0) {
 		// Real file, not empty buffer
-		long long maxSize = props.GetLongLong("max.file.size", 2000000000LL);
+		const long long maxSize = props.GetLongLong("max.file.size", 2000000000LL);
 		if (maxSize > 0 && fileSize > maxSize) {
 			const GUI::gui_string sSize = GUI::StringFromLongLong(fileSize);
 			const GUI::gui_string sMaxSize = GUI::StringFromLongLong(maxSize);
@@ -538,7 +540,7 @@ bool SciTEBase::Open(const FilePath &file, OpenFlags of) {
 			        "larger than the ^2 bytes limit set in the properties.\n"
 			        "Do you still want to open it?",
 			        absPath.AsInternal(), sSize.c_str(), sMaxSize.c_str());
-			MessageBoxChoice answer = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconWarning);
+			const MessageBoxChoice answer = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconWarning);
 			if (answer != mbYes) {
 				return false;
 			}
@@ -761,7 +763,7 @@ void SciTEBase::CheckReload() {
 						          "The file '^0' has been modified outside SciTE. Should it be reloaded?",
 						          FileNameExt().AsInternal());
 					}
-					MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconQuestion);
+					const MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconQuestion);
 					if (decision == mbYes) {
 						Open(filePath, static_cast<OpenFlags>(of | ofForceLoad));
 						DisplayAround(rf);
@@ -772,6 +774,18 @@ void SciTEBase::CheckReload() {
 				Open(filePath, static_cast<OpenFlags>(of | ofForceLoad));
 				DisplayAround(rf);
 			}
+		}  else if (newModTime == 0 && CurrentBuffer()->fileModTime != 0)  {
+			// Check if the file is deleted
+			CurrentBuffer()->fileModTime = 0;
+			CurrentBuffer()->fileModLastAsk = 0;
+			CurrentBuffer()->isDirty = true;
+			CheckMenus();
+			SetWindowName();
+			SetBuffersMenu();
+			GUI::gui_string msg = LocaliseMessage(
+						      "The file '^0' has been deleted.",
+						      filePath.AsInternal());
+			WindowMessageBox(wSciTE, msg, mbsOK);
 		}
 	}
 }
@@ -794,7 +808,7 @@ FilePath SciTEBase::SaveName(const char *ext) const {
 			dot--;
 		}
 		if (dot >= 0) {
-			int keepExt = props.GetInt("export.keep.ext");
+			const int keepExt = props.GetInt("export.keep.ext");
 			if (keepExt == 0) {
 				savePath.erase(dot);
 			} else if (keepExt == 2) {
@@ -826,7 +840,7 @@ SciTEBase::SaveResult SciTEBase::SaveIfUnsure(bool forceQuestion, SaveFlags sf)
 			} else {
 				msg = LocaliseMessage("Save changes to (Untitled)?");
 			}
-			MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNoCancel | mbsIconQuestion);
+			const MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNoCancel | mbsIconQuestion);
 			if (decision == mbYes) {
 				if (!Save(sf))
 					return saveCancelled;
@@ -892,9 +906,9 @@ SciTEBase::SaveResult SciTEBase::SaveIfUnsureForBuilt() {
 }
 
 void SciTEBase::StripTrailingSpaces() {
-	int maxLines = wEditor.Call(SCI_GETLINECOUNT);
+	const int maxLines = wEditor.Call(SCI_GETLINECOUNT);
 	for (int line = 0; line < maxLines; line++) {
-		int lineStart = wEditor.Call(SCI_POSITIONFROMLINE, line);
+		const int lineStart = wEditor.Call(SCI_POSITIONFROMLINE, line);
 		int lineEnd = wEditor.Call(SCI_GETLINEENDPOSITION, line);
 		int i = lineEnd - 1;
 		char ch = static_cast<char>(wEditor.Call(SCI_GETCHARAT, i));
@@ -994,7 +1008,7 @@ bool SciTEBase::SaveBuffer(const FilePath &saveName, SaveFlags sf) {
 					// Round down so only whole characters retrieved.
 					grabSize = wEditor.Call(SCI_POSITIONBEFORE, i + grabSize + 1) - i;
 					GetRange(wEditor, static_cast<int>(i), static_cast<int>(i + grabSize), &data[0]);
-					size_t written = convert.fwrite(&data[0], grabSize);
+					const size_t written = convert.fwrite(&data[0], grabSize);
 					if (written == 0) {
 						retVal = false;
 						break;
@@ -1050,12 +1064,12 @@ bool SciTEBase::Save(SaveFlags sf) {
 		if (props.GetInt("save.deletes.first")) {
 			filePath.Remove();
 		} else if (props.GetInt("save.check.modified.time")) {
-			time_t newModTime = filePath.ModifiedTime();
+			const time_t newModTime = filePath.ModifiedTime();
 			if ((newModTime != 0) && (CurrentBuffer()->fileModTime != 0) &&
 				(newModTime != CurrentBuffer()->fileModTime)) {
 				msg = LocaliseMessage("The file '^0' has been modified outside SciTE. Should it be saved?",
 					filePath.AsInternal());
-				MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconQuestion);
+				const MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconQuestion);
 				if (decision == mbNo) {
 					return false;
 				}
@@ -1078,7 +1092,7 @@ bool SciTEBase::Save(SaveFlags sf) {
 				CurrentBuffer()->failedSave = true;
 				msg = LocaliseMessage(
 					"Could not save file '^0'. Save under a different name?", filePath.AsInternal());
-				MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconWarning);
+				const MessageBoxChoice decision = WindowMessageBox(wSciTE, msg, mbsYesNo | mbsIconWarning);
 				if (decision == mbYes) {
 					return SaveAsDialog();
 				}
@@ -1087,7 +1101,24 @@ bool SciTEBase::Save(SaveFlags sf) {
 		}
 		return true;
 	} else {
-		return SaveAsDialog();
+		if (props.GetString("save.path.suggestion").length()) {
+			time_t t = time(NULL);
+			char timeBuff[15];
+			strftime(timeBuff, sizeof(timeBuff), "%Y%m%d%H%M%S",  localtime(&t));
+			PropSetFile propsSuggestion;
+			propsSuggestion.superPS = &props;  // Allow access to other settings
+			propsSuggestion.Set("TimeStamp", timeBuff);
+			propsSuggestion.Set("SciteUserHome", GetSciteUserHome().AsUTF8().c_str());
+			std::string savePathSuggestion = propsSuggestion.GetExpandedString("save.path.suggestion");
+			std::replace(savePathSuggestion.begin(), savePathSuggestion.end(), '\\', '/');  // To accept "\" on Unix
+			if (savePathSuggestion.size() > 0) {
+				filePath = FilePath(GUI::StringFromUTF8(savePathSuggestion)).NormalizePath();
+			}
+		}
+		bool ret = SaveAsDialog();
+		if (!ret)
+			filePath.Set(GUI_TEXT(""));
+		return ret;
 	}
 }
 
@@ -1106,7 +1137,7 @@ void SciTEBase::SaveAs(const GUI::gui_char *file, bool fixCase) {
 
 bool SciTEBase::SaveIfNotOpen(const FilePath &destFile, bool fixCase) {
 	FilePath absPath = destFile.AbsolutePath();
-	int index = buffers.GetDocumentByName(absPath, true /* excludeCurrent */);
+	const int index = buffers.GetDocumentByName(absPath, true /* excludeCurrent */);
 	if (index >= 0) {
 		GUI::gui_string msg = LocaliseMessage(
 			    "File '^0' is already open in another buffer.", destFile.AsInternal());
@@ -1142,7 +1173,7 @@ void SciTEBase::OpenFromStdin(bool UseOutputPane) {
 		wEditor.Call(SCI_CLEARALL);
 	}
 	size_t lenFile = fread(&data[0], 1, data.size(), stdin);
-	UniMode umCodingCookie = CodingCookieValue(&data[0], lenFile);
+	const UniMode umCodingCookie = CodingCookieValue(&data[0], lenFile);
 	while (lenFile > 0) {
 		lenFile = convert.convert(&data[0], lenFile);
 		if (UseOutputPane) {
@@ -1266,25 +1297,22 @@ public:
 };
 
 class FileReader {
-	BufferedFile *bf;
+	std::unique_ptr<BufferedFile> bf;
 	int lineNum;
 	bool lastWasCR;
 	std::string lineToCompare;
 	std::string lineToShow;
 	bool caseSensitive;
-	// Deleted so FileReader objects can not be copied
-	FileReader(const FileReader &) = delete;
 public:
-
 	FileReader(const FilePath &fPath, bool caseSensitive_) {
-		bf = new BufferedFile(fPath);
+		bf.reset(new BufferedFile(fPath));
 		lineNum = 0;
 		lastWasCR = false;
 		caseSensitive = caseSensitive_;
 	}
+	// Deleted so FileReader objects can not be copied.
+	FileReader(const FileReader &) = delete;
 	~FileReader() {
-		delete bf;
-		bf = NULL;
 	}
 	const char *Next() {
 		if (bf->Exhausted()) {
@@ -1305,11 +1333,7 @@ public:
 		lineNum++;
 		lineToCompare = lineToShow;
 		if (!caseSensitive) {
-			for (unsigned int j = 0; j < lineToCompare.length(); j++) {
-				if (lineToCompare[j] >= 'A' && lineToCompare[j] <= 'Z') {
-					lineToCompare[j] = static_cast<char>(lineToCompare[j] - 'A' + 'a');
-				}
-			}
+			LowerCaseAZ(lineToCompare);
 		}
 		return lineToCompare.c_str();
 	}
@@ -1341,12 +1365,11 @@ void SciTEBase::GrepRecursive(GrepFlags gf, const FilePath &baseDir, const char
 	FilePathSet directories;
 	FilePathSet files;
 	baseDir.List(directories, files);
-	size_t searchLength = strlen(searchString);
+	const size_t searchLength = strlen(searchString);
 	std::string os;
-	for (size_t i = 0; i < files.size(); i ++) {
+	for (const FilePath &fPath : files) {
 		if (jobQueue.Cancelled())
 			return;
-		FilePath fPath = files[i];
 		if (*fileTypes == '\0' || fPath.Matches(fileTypes)) {
 			//OutputAppendStringSynchronised(i->AsInternal());
 			//OutputAppendStringSynchronised("\n");
@@ -1386,8 +1409,7 @@ void SciTEBase::GrepRecursive(GrepFlags gf, const FilePath &baseDir, const char
 			OutputAppendStringSynchronised(os.c_str());
 		}
 	}
-	for (size_t j = 0; j < directories.size(); j++) {
-		FilePath fPath = directories[j];
+	for (const FilePath &fPath : directories) {
 		if ((gf & grepDot) || GrepIntoDirectory(fPath.Name())) {
 			GrepRecursive(gf, fPath, searchString, fileTypes);
 		}
