@@ -19,6 +19,7 @@
 #include <set>
 #include <map>
 #include <algorithm>
+#include <memory>
 
 #include "Scintilla.h"
 #include "SciLexer.h"
@@ -203,7 +204,7 @@ void SciTEBase::ReadLocalPropFile() {
 	props.Set("ChromeHighlight", "#FFFFFF");
 }
 
-Colour ColourOfProperty(PropSetFile &props, const char *key, Colour colourDefault) {
+Colour ColourOfProperty(const PropSetFile &props, const char *key, Colour colourDefault) {
 	std::string colour = props.GetExpandedString(key);
 	if (colour.length()) {
 		return ColourFromString(colour);
@@ -379,8 +380,8 @@ void SciTEBase::ReadAPI(const std::string &fileNameForExtension) {
 		std::vector<char> data;
 
 		// Load files into data
-		for (std::vector<std::string>::iterator it = vApiFileNames.begin(); it != vApiFileNames.end(); ++it) {
-			std::vector<char> contents = FilePath(GUI::StringFromUTF8(*it)).Read();
+		for (const std::string &vApiFileName : vApiFileNames) {
+			std::vector<char> contents = FilePath(GUI::StringFromUTF8(vApiFileName)).Read();
 			data.insert(data.end(), contents.begin(), contents.end());
 		}
 
@@ -507,8 +508,10 @@ static const char *propertiesToForward[] = {
 	"lexer.python.keywords2.no.sub.identifiers",
 	"lexer.python.literals.binary",
 	"lexer.python.strings.b",
+	"lexer.python.strings.f",
 	"lexer.python.strings.over.newline",
 	"lexer.python.strings.u",
+	"lexer.python.unicode.identifiers",
 	"lexer.rust.fold.at.else",
 	"lexer.sql.allow.dotted.word",
 	"lexer.sql.backticks.identifier",
@@ -780,6 +783,10 @@ void SciTEBase::ReadProperties() {
 	}
 	imeAutoComplete = props.GetInt("ime.autocomplete", 0) == 1;
 
+	const int accessibility = props.GetInt("accessibility", 1);
+	wEditor.Call(SCI_SETACCESSIBILITY, accessibility);
+	wOutput.Call(SCI_SETACCESSIBILITY, accessibility);
+
 	wrapStyle = props.GetInt("wrap.style", SC_WRAP_WORD);
 
 	CallChildren(SCI_SETCARETFORE,
@@ -798,7 +805,7 @@ void SciTEBase::ReadProperties() {
 	wEditor.Call(SCI_SETCARETWIDTH, props.GetInt("caret.width", 1));
 	wOutput.Call(SCI_SETCARETWIDTH, props.GetInt("caret.width", 1));
 
-	std::string caretLineBack = props.GetString("caret.line.back");
+	std::string caretLineBack = props.GetExpandedString("caret.line.back");
 	if (caretLineBack.length()) {
 		wEditor.Call(SCI_SETCARETLINEVISIBLE, 1);
 		wEditor.Call(SCI_SETCARETLINEBACK, ColourFromString(caretLineBack));
@@ -852,13 +859,13 @@ void SciTEBase::ReadProperties() {
 	wEditor.Call(SCI_SETEDGECOLOUR,
 	           ColourOfProperty(props, "edge.colour", ColourRGB(0xff, 0xda, 0xda)));
 
-	std::string selFore = props.GetString("selection.fore");
+	std::string selFore = props.GetExpandedString("selection.fore");
 	if (selFore.length()) {
 		CallChildren(SCI_SETSELFORE, 1, ColourFromString(selFore));
 	} else {
 		CallChildren(SCI_SETSELFORE, 0, 0);
 	}
-	std::string selBack = props.GetString("selection.back");
+	std::string selBack = props.GetExpandedString("selection.back");
 	if (selBack.length()) {
 		CallChildren(SCI_SETSELBACK, 1, ColourFromString(selBack));
 	} else {
@@ -878,29 +885,29 @@ void SciTEBase::ReadProperties() {
 	if (selAdditionalBack.length()) {
 		CallChildren(SCI_SETADDITIONALSELBACK, ColourFromString(selAdditionalBack));
 	}
-	int selectionAdditionalAlpha = (selectionAlpha == SC_ALPHA_NOALPHA) ? SC_ALPHA_NOALPHA : selectionAlpha / 2;
+	const int selectionAdditionalAlpha = (selectionAlpha == SC_ALPHA_NOALPHA) ? SC_ALPHA_NOALPHA : selectionAlpha / 2;
 	CallChildren(SCI_SETADDITIONALSELALPHA, props.GetInt("selection.additional.alpha", selectionAdditionalAlpha));
 
-	std::string foldColour = props.GetString("fold.margin.colour");
+	foldColour = props.GetExpandedString("fold.margin.colour");
 	if (foldColour.length()) {
 		CallChildren(SCI_SETFOLDMARGINCOLOUR, 1, ColourFromString(foldColour));
 	} else {
 		CallChildren(SCI_SETFOLDMARGINCOLOUR, 0, 0);
 	}
-	std::string foldHiliteColour = props.GetString("fold.margin.highlight.colour");
+	foldHiliteColour = props.GetExpandedString("fold.margin.highlight.colour");
 	if (foldHiliteColour.length()) {
 		CallChildren(SCI_SETFOLDMARGINHICOLOUR, 1, ColourFromString(foldHiliteColour));
 	} else {
 		CallChildren(SCI_SETFOLDMARGINHICOLOUR, 0, 0);
 	}
 
-	std::string whitespaceFore = props.GetString("whitespace.fore");
+	std::string whitespaceFore = props.GetExpandedString("whitespace.fore");
 	if (whitespaceFore.length()) {
 		CallChildren(SCI_SETWHITESPACEFORE, 1, ColourFromString(whitespaceFore));
 	} else {
 		CallChildren(SCI_SETWHITESPACEFORE, 0, 0);
 	}
-	std::string whitespaceBack = props.GetString("whitespace.back");
+	std::string whitespaceBack = props.GetExpandedString("whitespace.back");
 	if (whitespaceBack.length()) {
 		CallChildren(SCI_SETWHITESPACEBACK, 1, ColourFromString(whitespaceBack));
 	} else {
@@ -1004,9 +1011,7 @@ void SciTEBase::ReadProperties() {
 	wEditor.Call(SCI_SETBUFFEREDDRAW, bufferedDraw);
 	wOutput.Call(SCI_SETBUFFEREDDRAW, bufferedDraw);
 
-	int phasesDraw = props.GetInt("phases.draw", -1);
-	if (phasesDraw < 0 || phasesDraw > SC_PHASES_MULTIPLE)
-		phasesDraw = props.GetInt("two.phase.draw", 1) ? SC_PHASES_TWO : SC_PHASES_ONE;
+	const int phasesDraw = props.GetInt("phases.draw", 1);
 	wEditor.Call(SCI_SETPHASESDRAW, phasesDraw);
 	wOutput.Call(SCI_SETPHASESDRAW, phasesDraw);
 
@@ -1050,22 +1055,23 @@ void SciTEBase::ReadProperties() {
 	blockStart = GetStyleAndWords("block.start.");
 	blockEnd = GetStyleAndWords("block.end.");
 
-	struct {
+	struct PropToPPC {
 		const char *propName;
 		PreProcKind ppc;
-	} propToPPC[] = {
+	};
+	PropToPPC propToPPC[] = {
 		{"preprocessor.start.", ppcStart},
 		{"preprocessor.middle.", ppcMiddle},
 		{"preprocessor.end.", ppcEnd},
 	};
-	std::string list = props.GetNewExpandString("preprocessor.symbol.", fileNameForExtension.c_str());
-	preprocessorSymbol = list.empty() ? 0 : list[0];
+	const std::string ppSymbol = props.GetNewExpandString("preprocessor.symbol.", fileNameForExtension.c_str());
+	preprocessorSymbol = ppSymbol.empty() ? 0 : ppSymbol[0];
 	preprocOfString.clear();
-	for (size_t iPreproc = 0; iPreproc < ELEMENTS(propToPPC); iPreproc++) {
-		list = props.GetNewExpandString(propToPPC[iPreproc].propName, fileNameForExtension.c_str());
-		std::vector<std::string> words = StringSplit(list, ' ');
-		for (std::vector<std::string>::iterator it = words.begin(); it != words.end(); ++it) {
-			preprocOfString[*it] = propToPPC[iPreproc].ppc;
+	for (const PropToPPC &preproc : propToPPC) {
+		const std::string list = props.GetNewExpandString(preproc.propName, fileNameForExtension.c_str());
+		const std::vector<std::string> words = StringSplit(list, ' ');
+		for (const std::string &word : words) {
+			preprocOfString[word] = preproc.ppc;
 		}
 	}
 
@@ -1151,88 +1157,122 @@ void SciTEBase::ReadProperties() {
 	wEditor.Call(SCI_SETMARGINMASKN, 2, SC_MASK_FOLDERS);
 	wEditor.Call(SCI_SETMARGINSENSITIVEN, 2, 1);
 
-	// Enable/disable highlight for current folding bloc (smallest one that contains the caret)
+	// Define foreground (outline) and background (fill) color of folds
+	const int foldSymbols = props.GetInt("fold.symbols");
+	std::string foldFore = props.GetExpandedString("fold.fore");
+	if (foldFore.length() == 0) {
+		// Set default colour for outline
+		switch (foldSymbols) {
+		case 0: // Arrows
+			foldFore = "#000000";
+			break;
+		case 1: // + -
+			foldFore = "#FFFFFF";
+			break;
+		case 2: // Circles
+			foldFore = "#404040";
+			break;
+		case 3: // Squares
+			foldFore = "#808080";
+			break;
+		}
+	}
+	Colour colourFoldFore = ColourFromString(foldFore);
+
+	std::string foldBack = props.GetExpandedString("fold.back");
+	// Set default colour for fill
+	if (foldBack.length() == 0) {
+		switch (foldSymbols) {
+		case 0:
+		case 1:
+			foldBack = "#000000";
+			break;
+		case 2:
+		case 3:
+			foldBack = "#FFFFFF";
+			break;
+		}
+	}
+	Colour colourFoldBack = ColourFromString(foldBack);
+
+	// Enable/disable highlight for current folding block (smallest one that contains the caret)
 	int isHighlightEnabled = props.GetInt("fold.highlight", 0);
 	// Define the colour of highlight
-	std::string foldBlockHighlight = props.GetString("fold.highlight.colour");
-	if (foldBlockHighlight.length() == 0) {
-		//Set default colour for highlight
-		foldBlockHighlight = "#FF0000";
-	}
-	Colour colourFoldBlockHighlight = ColourFromString(foldBlockHighlight);
-	switch (props.GetInt("fold.symbols")) {
+	Colour colourFoldBlockHighlight = ColourOfProperty(props, "fold.highlight.colour", ColourRGB(0xFF, 0, 0));
+
+	switch (foldSymbols) {
 	case 0:
 		// Arrow pointing right for contracted folders, arrow pointing down for expanded
 		DefineMarker(SC_MARKNUM_FOLDEROPEN, SC_MARK_ARROWDOWN,
-					 ColourRGB(0, 0, 0), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+					 colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDER, SC_MARK_ARROW,
-					 ColourRGB(0, 0, 0), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+					 colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERSUB, SC_MARK_EMPTY,
-					 ColourRGB(0, 0, 0), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+					 colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERTAIL, SC_MARK_EMPTY,
-					 ColourRGB(0, 0, 0), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+					 colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDEREND, SC_MARK_EMPTY,
-					 ColourRGB(0xff, 0xff, 0xff), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+					 colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDEROPENMID, SC_MARK_EMPTY,
-					 ColourRGB(0xff, 0xff, 0xff), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+					 colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_EMPTY,
-					 ColourRGB(0xff, 0xff, 0xff), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+					 colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		// The highlight is disabled for arrow.
 		wEditor.Call(SCI_MARKERENABLEHIGHLIGHT, false);
 		break;
 	case 1:
 		// Plus for contracted folders, minus for expanded
 		DefineMarker(SC_MARKNUM_FOLDEROPEN, SC_MARK_MINUS,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+		             colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDER, SC_MARK_PLUS,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+		             colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERSUB, SC_MARK_EMPTY,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+		             colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERTAIL, SC_MARK_EMPTY,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+		             colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDEREND, SC_MARK_EMPTY,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+		             colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDEROPENMID, SC_MARK_EMPTY,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+		             colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_EMPTY,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0, 0, 0), colourFoldBlockHighlight);
+		             colourFoldFore, colourFoldBack, colourFoldBlockHighlight);
 		// The highlight is disabled for plus/minus.
 		wEditor.Call(SCI_MARKERENABLEHIGHLIGHT, false);
 		break;
 	case 2:
 		// Like a flattened tree control using circular headers and curved joins
 		DefineMarker(SC_MARKNUM_FOLDEROPEN, SC_MARK_CIRCLEMINUS,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x40, 0x40, 0x40), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDER, SC_MARK_CIRCLEPLUS,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x40, 0x40, 0x40), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERSUB, SC_MARK_VLINE,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x40, 0x40, 0x40), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERTAIL, SC_MARK_LCORNERCURVE,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x40, 0x40, 0x40), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDEREND, SC_MARK_CIRCLEPLUSCONNECTED,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x40, 0x40, 0x40), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDEROPENMID, SC_MARK_CIRCLEMINUSCONNECTED,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x40, 0x40, 0x40), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_TCORNERCURVE,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x40, 0x40, 0x40), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		wEditor.Call(SCI_MARKERENABLEHIGHLIGHT, isHighlightEnabled);
 		break;
 	case 3:
 		// Like a flattened tree control using square headers
 		DefineMarker(SC_MARKNUM_FOLDEROPEN, SC_MARK_BOXMINUS,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x80, 0x80, 0x80), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDER, SC_MARK_BOXPLUS,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x80, 0x80, 0x80), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERSUB, SC_MARK_VLINE,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x80, 0x80, 0x80), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERTAIL, SC_MARK_LCORNER,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x80, 0x80, 0x80), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDEREND, SC_MARK_BOXPLUSCONNECTED,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x80, 0x80, 0x80), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDEROPENMID, SC_MARK_BOXMINUSCONNECTED,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x80, 0x80, 0x80), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		DefineMarker(SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_TCORNER,
-		             ColourRGB(0xff, 0xff, 0xff), ColourRGB(0x80, 0x80, 0x80), colourFoldBlockHighlight);
+		             colourFoldBack, colourFoldFore, colourFoldBlockHighlight);
 		wEditor.Call(SCI_MARKERENABLEHIGHLIGHT, isHighlightEnabled);
 		break;
 	}
@@ -1275,11 +1315,11 @@ void SciTEBase::ReadProperties() {
 
 	currentWordHighlight.isEnabled = props.GetInt("highlight.current.word", 0) == 1;
 	if (currentWordHighlight.isEnabled) {
-		const std::string highlightCurrentWordIndicatorString = props.GetString("highlight.current.word.indicator");
+		const std::string highlightCurrentWordIndicatorString = props.GetExpandedString("highlight.current.word.indicator");
 		IndicatorDefinition highlightCurrentWordIndicator(highlightCurrentWordIndicatorString.c_str());
 		if (highlightCurrentWordIndicatorString.length() == 0) {
 			highlightCurrentWordIndicator.style = INDIC_ROUNDBOX;
-			std::string highlightCurrentWordColourString = props.GetString("highlight.current.word.colour");
+			std::string highlightCurrentWordColourString = props.GetExpandedString("highlight.current.word.colour");
 			if (highlightCurrentWordColourString.length() == 0) {
 				// Set default colour for highlight.
 				highlightCurrentWordColourString = "#A0A000";
@@ -1373,14 +1413,14 @@ void SciTEBase::ReadFontProperties() {
 		SetStyleBlock(wEditor, "error", diagnosticStyleStart, diagnosticStyleStart+diagnosticStyles-1);
 	}
 
-	int diffToSecondary = static_cast<int>(wEditor.Call(SCI_DISTANCETOSECONDARYSTYLES));
-	for (unsigned int baseStyle=0; baseStyle<subStyleBases.size(); baseStyle++) {
-		int subStylesStart = wEditor.Call(SCI_GETSUBSTYLESSTART, subStyleBases[baseStyle]);
-		int subStylesLength = wEditor.Call(SCI_GETSUBSTYLESLENGTH, subStyleBases[baseStyle]);
+	const int diffToSecondary = static_cast<int>(wEditor.Call(SCI_DISTANCETOSECONDARYSTYLES));
+	for (const char subStyleBase : subStyleBases) {
+		const int subStylesStart = wEditor.Call(SCI_GETSUBSTYLESSTART, subStyleBase);
+		const int subStylesLength = wEditor.Call(SCI_GETSUBSTYLESLENGTH, subStyleBase);
 		for (int subStyle=0; subStyle<subStylesLength; subStyle++) {
 			for (int active=0; active<(diffToSecondary?2:1); active++) {
 				int activity = active * diffToSecondary;
-				sprintf(key, "style.%s.%0d.%0d", languageName, subStyleBases[baseStyle] + activity, subStyle+1);
+				sprintf(key, "style.%s.%0d.%0d", languageName, subStyleBase + activity, subStyle+1);
 				sval = props.GetNewExpandString(key);
 				SetOneStyle(wEditor, subStylesStart + subStyle + activity, sval.c_str());
 			}
@@ -1509,9 +1549,9 @@ void SciTEBase::ReadLocalization() {
 
 void SciTEBase::ReadPropertiesInitial() {
 	SetPropertiesInitial();
-	int sizeHorizontal = props.GetInt("output.horizontal.size", 0);
-	int sizeVertical = props.GetInt("output.vertical.size", 0);
-	int hideOutput = props.GetInt("output.initial.hide", 0);
+	const int sizeHorizontal = props.GetInt("output.horizontal.size", 0);
+	const int sizeVertical = props.GetInt("output.vertical.size", 0);
+	const int hideOutput = props.GetInt("output.initial.hide", 0);
 	if ((!splitVertical && (sizeVertical > 0) && (heightOutput < sizeVertical)) ||
 		(splitVertical && (sizeHorizontal > 0) && (heightOutput < sizeHorizontal))) {
 		previousHeightOutput = splitVertical ? sizeHorizontal : sizeVertical;
@@ -1549,7 +1589,7 @@ void SciTEBase::ReadPropertiesInitial() {
 	// load the user defined short cut props
 	std::string shortCutProp = props.GetNewExpandString("user.shortcuts");
 	if (shortCutProp.length()) {
-		size_t pipes = std::count(shortCutProp.begin(), shortCutProp.end(), '|');
+		const size_t pipes = std::count(shortCutProp.begin(), shortCutProp.end(), '|');
 		std::replace(shortCutProp.begin(), shortCutProp.end(), '|', '\0');
 		const char *sShortCutProp = shortCutProp.c_str();
 		for (size_t item = 0; item < pipes/2; item++) {
@@ -1632,7 +1672,7 @@ void SciTEBase::OpenProperties(int propsFile) {
 		}
 	case IDM_OPENDIRECTORYPROPERTIES: {
 			propfile = GetDirectoryPropertiesFileName();
-			bool alreadyExists = propfile.Exists();
+			const bool alreadyExists = propfile.Exists();
 			Open(propfile, ofQuiet);
 			if (!alreadyExists)
 				SaveAsDialog();
@@ -1647,6 +1687,15 @@ int SciTEBase::GetMenuCommandAsInt(std::string commandName) {
 	if (i != -1) {
 		return IFaceTable::constants[i].value;
 	}
+
+	// Check also for a SCI command, as long as it has no parameters
+	i = IFaceTable::FindFunctionByConstantName(commandName.c_str());
+	if (i != -1 &&
+		IFaceTable::functions[i].paramType[0] == iface_void &&
+		IFaceTable::functions[i].paramType[1] == iface_void) {
+		return IFaceTable::functions[i].value;
+	}
+
 	// Otherwise we might have entered a number as command to access a "SCI_" command
 	return atoi(commandName.c_str());
 }
