@@ -18,11 +18,7 @@ class LexState;
 
 /**
  */
-class ScintillaBase : public Editor {
-	// Private so ScintillaBase objects can not be copied
-	ScintillaBase(const ScintillaBase &);
-	ScintillaBase &operator=(const ScintillaBase &);
-
+class ScintillaBase : public Editor, IListBoxDelegate {
 protected:
 	/** Enumeration of commands and child windows. */
 	enum {
@@ -38,7 +34,9 @@ protected:
 		idcmdSelectAll=16
 	};
 
-	bool displayPopupMenu;
+	enum { maxLenInputIME = 200 };
+
+	int displayPopupMenu;
 	Menu popup;
 	AutoComplete ac;
 
@@ -46,6 +44,7 @@ protected:
 
 	int listType;			///< 0 is an autocomplete list
 	int maxListWidth;		/// Maximum width of list, in average character widths
+	int multiAutoCMode; /// Mode for autocompleting when multiple selections are present
 
 #ifdef SCI_LEXER
 	LexState *DocumentLexState();
@@ -55,42 +54,48 @@ protected:
 #endif
 
 	ScintillaBase();
+	// Deleted so ScintillaBase objects can not be copied.
+	explicit ScintillaBase(const ScintillaBase &) = delete;
+	ScintillaBase &operator=(const ScintillaBase &) = delete;
 	virtual ~ScintillaBase();
-	virtual void Initialise() = 0;
-	virtual void Finalise() = 0;
+	void Initialise() override {}
+	void Finalise() override;
 
-	virtual void AddCharUTF(char *s, unsigned int len, bool treatAsDBCS=false);
+	void AddCharUTF(const char *s, unsigned int len, bool treatAsDBCS=false) override;
 	void Command(int cmdId);
-	virtual void CancelModes();
-	virtual int KeyCommand(unsigned int iMessage);
+	void CancelModes() override;
+	int KeyCommand(unsigned int iMessage) override;
 
-	void AutoCompleteInsert(Position startPos, int removeLen, const char *text, int textLen);
+	void AutoCompleteInsert(Sci::Position startPos, int removeLen, const char *text, int textLen);
 	void AutoCompleteStart(int lenEntered, const char *list);
 	void AutoCompleteCancel();
 	void AutoCompleteMove(int delta);
-	int AutoCompleteGetCurrent();
-	int AutoCompleteGetCurrentText(char *buffer);
+	int AutoCompleteGetCurrent() const;
+	int AutoCompleteGetCurrentText(char *buffer) const;
 	void AutoCompleteCharacterAdded(char ch);
 	void AutoCompleteCharacterDeleted();
-	void AutoCompleteCompleted();
+	void AutoCompleteCompleted(char ch, unsigned int completionMethod);
 	void AutoCompleteMoveToCurrentWord();
-	static void AutoCompleteDoubleClick(void *p);
+	void AutoCompleteSelection();
+	void ListNotify(ListBoxEvent *plbe) override;
 
 	void CallTipClick();
 	void CallTipShow(Point pt, const char *defn);
 	virtual void CreateCallTipWindow(PRectangle rc) = 0;
 
 	virtual void AddToPopUp(const char *label, int cmd=0, bool enabled=true) = 0;
+	bool ShouldDisplayPopup(Point ptInWindowCoordinates) const;
 	void ContextMenu(Point pt);
 
-	virtual void ButtonDown(Point pt, unsigned int curTime, bool shift, bool ctrl, bool alt);
+	void ButtonDownWithModifiers(Point pt, unsigned int curTime, int modifiers) override;
+	void RightButtonDownWithModifiers(Point pt, unsigned int curTime, int modifiers) override;
 
-	void NotifyStyleToNeeded(int endStyleNeeded);
-	void NotifyLexerChanged(Document *doc, void *userData);
+	void NotifyStyleToNeeded(Sci::Position endStyleNeeded) override;
+	void NotifyLexerChanged(Document *doc, void *userData) override;
 
 public:
 	// Public so scintilla_send_message can use it
-	virtual sptr_t WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam);
+	sptr_t WndProc(unsigned int iMessage, uptr_t wParam, sptr_t lParam) override;
 };
 
 #ifdef SCI_NAMESPACE
