@@ -5,7 +5,7 @@
 // Copyright 2009 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
-#include <stdlib.h>
+#include <cstdlib>
 
 #include <stdexcept>
 #include <vector>
@@ -22,10 +22,10 @@
 using namespace Scintilla;
 #endif
 
-void SelectionPosition::MoveForInsertDelete(bool insertion, int startChange, int length) {
+void SelectionPosition::MoveForInsertDelete(bool insertion, Sci::Position startChange, Sci::Position length) {
 	if (insertion) {
 		if (position == startChange) {
-			int virtualLengthRemove = std::min(length, virtualSpace);
+			Sci::Position virtualLengthRemove = std::min(length, virtualSpace);
 			virtualSpace -= virtualLengthRemove;
 			position += virtualLengthRemove;
 		} else if (position > startChange) {
@@ -36,7 +36,7 @@ void SelectionPosition::MoveForInsertDelete(bool insertion, int startChange, int
 			virtualSpace = 0;
 		}
 		if (position > startChange) {
-			int endDeletion = startChange + length;
+			const Sci::Position endDeletion = startChange + length;
 			if (position > endDeletion) {
 				position -= length;
 			} else {
@@ -75,7 +75,7 @@ bool SelectionPosition::operator >=(const SelectionPosition &other) const {
 		return *this > other;
 }
 
-int SelectionRange::Length() const {
+Sci::Position SelectionRange::Length() const {
 	if (anchor > caret) {
 		return anchor.Position() - caret.Position();
 	} else {
@@ -83,12 +83,12 @@ int SelectionRange::Length() const {
 	}
 }
 
-void SelectionRange::MoveForInsertDelete(bool insertion, int startChange, int length) {
+void SelectionRange::MoveForInsertDelete(bool insertion, Sci::Position startChange, Sci::Position length) {
 	caret.MoveForInsertDelete(insertion, startChange, length);
 	anchor.MoveForInsertDelete(insertion, startChange, length);
 }
 
-bool SelectionRange::Contains(int pos) const {
+bool SelectionRange::Contains(Sci::Position pos) const {
 	if (anchor > caret)
 		return (pos >= caret.Position()) && (pos <= anchor.Position());
 	else
@@ -102,7 +102,7 @@ bool SelectionRange::Contains(SelectionPosition sp) const {
 		return (sp >= anchor) && (sp <= caret);
 }
 
-bool SelectionRange::ContainsCharacter(int posCharacter) const {
+bool SelectionRange::ContainsCharacter(Sci::Position posCharacter) const {
 	if (anchor > caret)
 		return (posCharacter >= caret.Position()) && (posCharacter < anchor.Position());
 	else
@@ -126,9 +126,13 @@ SelectionSegment SelectionRange::Intersect(SelectionSegment check) const {
 	}
 }
 
+void SelectionRange::Swap() {
+	std::swap(caret, anchor);
+}
+
 bool SelectionRange::Trim(SelectionRange range) {
-	SelectionPosition startRange = range.Start();
-	SelectionPosition endRange = range.End();
+	const SelectionPosition startRange = range.Start();
+	const SelectionPosition endRange = range.End();
 	SelectionPosition start = Start();
 	SelectionPosition end = End();
 	PLATFORM_ASSERT(start <= end);
@@ -164,7 +168,7 @@ bool SelectionRange::Trim(SelectionRange range) {
 // If range is all virtual collapse to start of virtual space
 void SelectionRange::MinimizeVirtualSpace() {
 	if (caret.Position() == anchor.Position()) {
-		int virtualSpace = caret.VirtualSpace();
+		Sci::Position virtualSpace = caret.VirtualSpace();
 		if (virtualSpace > anchor.VirtualSpace())
 			virtualSpace = anchor.VirtualSpace();
 		caret.SetVirtualSpace(virtualSpace);
@@ -183,11 +187,11 @@ bool Selection::IsRectangular() const {
 	return (selType == selRectangle) || (selType == selThin);
 }
 
-int Selection::MainCaret() const {
+Sci::Position Selection::MainCaret() const {
 	return ranges[mainRange].caret.Position();
 }
 
-int Selection::MainAnchor() const {
+Sci::Position Selection::MainAnchor() const {
 	return ranges[mainRange].anchor.Position();
 }
 
@@ -262,8 +266,8 @@ void Selection::SetMoveExtends(bool moveExtends_) {
 }
 
 bool Selection::Empty() const {
-	for (size_t i=0; i<ranges.size(); i++) {
-		if (!ranges[i].Empty())
+	for (const SelectionRange &range : ranges) {
+		if (!range.Empty())
 			return false;
 	}
 	return true;
@@ -271,26 +275,26 @@ bool Selection::Empty() const {
 
 SelectionPosition Selection::Last() const {
 	SelectionPosition lastPosition;
-	for (size_t i=0; i<ranges.size(); i++) {
-		if (lastPosition < ranges[i].caret)
-			lastPosition = ranges[i].caret;
-		if (lastPosition < ranges[i].anchor)
-			lastPosition = ranges[i].anchor;
+	for (const SelectionRange &range : ranges) {
+		if (lastPosition < range.caret)
+			lastPosition = range.caret;
+		if (lastPosition < range.anchor)
+			lastPosition = range.anchor;
 	}
 	return lastPosition;
 }
 
-int Selection::Length() const {
-	int len = 0;
-	for (size_t i=0; i<ranges.size(); i++) {
-		len += ranges[i].Length();
+Sci::Position Selection::Length() const {
+	Sci::Position len = 0;
+	for (const SelectionRange &range : ranges) {
+		len += range.Length();
 	}
 	return len;
 }
 
-void Selection::MovePositions(bool insertion, int startChange, int length) {
-	for (size_t i=0; i<ranges.size(); i++) {
-		ranges[i].MoveForInsertDelete(insertion, startChange, length);
+void Selection::MovePositions(bool insertion, Sci::Position startChange, Sci::Position length) {
+	for (SelectionRange &range : ranges) {
+		range.MoveForInsertDelete(insertion, startChange, length);
 	}
 	if (selType == selRectangle) {
 		rangeRectangular.MoveForInsertDelete(insertion, startChange, length);
@@ -372,7 +376,7 @@ void Selection::CommitTentative() {
 	tentativeMain = false;
 }
 
-int Selection::CharacterInSelection(int posCharacter) const {
+int Selection::CharacterInSelection(Sci::Position posCharacter) const {
 	for (size_t i=0; i<ranges.size(); i++) {
 		if (ranges[i].ContainsCharacter(posCharacter))
 			return i == mainRange ? 1 : 2;
@@ -380,7 +384,7 @@ int Selection::CharacterInSelection(int posCharacter) const {
 	return 0;
 }
 
-int Selection::InSelectionForEOL(int pos) const {
+int Selection::InSelectionForEOL(Sci::Position pos) const {
 	for (size_t i=0; i<ranges.size(); i++) {
 		if (!ranges[i].Empty() && (pos > ranges[i].Start().Position()) && (pos <= ranges[i].End().Position()))
 			return i == mainRange ? 1 : 2;
@@ -388,13 +392,13 @@ int Selection::InSelectionForEOL(int pos) const {
 	return 0;
 }
 
-int Selection::VirtualSpaceFor(int pos) const {
-	int virtualSpace = 0;
-	for (size_t i=0; i<ranges.size(); i++) {
-		if ((ranges[i].caret.Position() == pos) && (virtualSpace < ranges[i].caret.VirtualSpace()))
-			virtualSpace = ranges[i].caret.VirtualSpace();
-		if ((ranges[i].anchor.Position() == pos) && (virtualSpace < ranges[i].anchor.VirtualSpace()))
-			virtualSpace = ranges[i].anchor.VirtualSpace();
+Sci::Position Selection::VirtualSpaceFor(Sci::Position pos) const {
+	Sci::Position virtualSpace = 0;
+	for (const SelectionRange &range : ranges) {
+		if ((range.caret.Position() == pos) && (virtualSpace < range.caret.VirtualSpace()))
+			virtualSpace = range.caret.VirtualSpace();
+		if ((range.anchor.Position() == pos) && (virtualSpace < range.anchor.VirtualSpace()))
+			virtualSpace = range.anchor.VirtualSpace();
 	}
 	return virtualSpace;
 }
