@@ -93,7 +93,7 @@ std::string StdStringFromDouble(double d, int precision) {
 	return std::string(number);
 }
 
-static int LowerCaseAZ(int c) {
+static char LowerCaseAZChar(char c) {
 	if (c >= 'A' && c <= 'Z') {
 		return c - 'A' + 'a';
 	} else {
@@ -102,14 +102,14 @@ static int LowerCaseAZ(int c) {
 }
 
 void LowerCaseAZ(std::string &s) {
-	std::transform(s.begin(), s.end(), s.begin(), std::ptr_fun<int, int>(LowerCaseAZ));
+	std::transform(s.begin(), s.end(), s.begin(), LowerCaseAZChar);
 }
 
 int CompareNoCase(const char *a, const char *b) {
 	while (*a && *b) {
 		if (*a != *b) {
-			char upperA = MakeUpperCase(*a);
-			char upperB = MakeUpperCase(*b);
+			const char upperA = MakeUpperCase(*a);
+			const char upperB = MakeUpperCase(*b);
 			if (upperA != upperB)
 				return upperA - upperB;
 		}
@@ -166,56 +166,39 @@ unsigned int UTF32Character(const unsigned char *utf8) {
 
 /**
  * Convert a string into C string literal form using \a, \b, \f, \n, \r, \t, \v, and \ooo.
- * The return value is a newly allocated character array containing the result.
- * 4 bytes are allocated for each byte of the input because that is the maximum
- * expansion needed when all of the input needs to be output using the octal form.
- * The return value should be deleted with delete[].
  */
-char *Slash(const char *s, bool quoteQuotes) {
-	char *oRet = new char[strlen(s) * 4 + 1];
-	char *o = oRet;
-	while (*s) {
-		if (*s == '\a') {
-			*o++ = '\\';
-			*o++ = 'a';
-		} else if (*s == '\b') {
-			*o++ = '\\';
-			*o++ = 'b';
-		} else if (*s == '\f') {
-			*o++ = '\\';
-			*o++ = 'f';
-		} else if (*s == '\n') {
-			*o++ = '\\';
-			*o++ = 'n';
-		} else if (*s == '\r') {
-			*o++ = '\\';
-			*o++ = 'r';
-		} else if (*s == '\t') {
-			*o++ = '\\';
-			*o++ = 't';
-		} else if (*s == '\v') {
-			*o++ = '\\';
-			*o++ = 'v';
-		} else if (*s == '\\') {
-			*o++ = '\\';
-			*o++ = '\\';
-		} else if (quoteQuotes && (*s == '\'')) {
-			*o++ = '\\';
-			*o++ = '\'';
-		} else if (quoteQuotes && (*s == '\"')) {
-			*o++ = '\\';
-			*o++ = '\"';
-		} else if (isascii(*s) && (*s < ' ')) {
-			*o++ = '\\';
-			*o++ = static_cast<char>((*s >> 6) + '0');
-			*o++ = static_cast<char>((*s >> 3) + '0');
-			*o++ = static_cast<char>((*s & 0x7) + '0');
+std::string Slash(const std::string &s, bool quoteQuotes) {
+	std::string oRet;
+	for (char ch : s) {
+		if (ch == '\a') {
+			oRet.append("\\a");
+		} else if (ch == '\b') {
+			oRet.append("\\b");
+		} else if (ch == '\f') {
+			oRet.append("\\f");
+		} else if (ch == '\n') {
+			oRet.append("\\n");
+		} else if (ch == '\r') {
+			oRet.append("\\r");
+		} else if (ch == '\t') {
+			oRet.append("\\t");
+		} else if (ch == '\v') {
+			oRet.append("\\v");
+		} else if (ch == '\\') {
+			oRet.append("\\\\");
+		} else if (quoteQuotes && (ch == '\'')) {
+			oRet.append("\\\'");
+		} else if (quoteQuotes && (ch == '\"')) {
+			oRet.append("\\\"");
+		} else if (IsASCII(ch) && (ch < ' ')) {
+			oRet.push_back('\\');
+			oRet.push_back(static_cast<char>((ch >> 6) + '0'));
+			oRet.push_back(static_cast<char>((ch >> 3) + '0'));
+			oRet.push_back(static_cast<char>((ch & 0x7) + '0'));
 		} else {
-			*o++ = *s;
+			oRet.push_back(ch);
 		}
-		s++;
 	}
-	*o = '\0';
 	return oRet;
 }
 
@@ -246,7 +229,7 @@ static int GetHexaDigit(char ch) {
  * Convert C style \a, \b, \f, \n, \r, \t, \v, \ooo and \xhh into their indicated characters.
  */
 unsigned int UnSlash(char *s) {
-	char *sStart = s;
+	const char *sStart = s;
 	char *o = s;
 
 	while (*s) {
@@ -320,7 +303,7 @@ std::string UnSlashString(const char *s) {
  * This is used to get control characters into the regular expresion engine.
  */
 static unsigned int UnSlashLowOctal(char *s) {
-	char *sStart = s;
+	const char *sStart = s;
 	char *o = s;
 	while (*s) {
 		if ((s[0] == '\\') && (s[1] == '0') && IsOctalDigit(s[2]) && IsOctalDigit(s[3])) {
