@@ -27,8 +27,8 @@ static inline bool IsASpace(unsigned int ch) {
 static int CompareNCaseInsensitive(const char *a, const char *b, size_t len) {
 	while (*a && *b && len) {
 		if (*a != *b) {
-			char upperA = MakeUpperCase(*a);
-			char upperB = MakeUpperCase(*b);
+			const char upperA = MakeUpperCase(*a);
+			const char upperB = MakeUpperCase(*b);
 			if (upperA != upperB)
 				return upperA - upperB;
 		}
@@ -52,10 +52,7 @@ static std::vector<char *> ArrayFromStringList(char *stringList, bool onlyLineEn
 	int words = 0;
 	// For rapid determination of whether a character is a separator, build
 	// a look up table.
-	bool wordSeparator[256];
-	for (int i=0; i<256; i++) {
-		wordSeparator[i] = false;
-	}
+	bool wordSeparator[256] = {};
 	wordSeparator[static_cast<unsigned int>('\r')] = true;
 	wordSeparator[static_cast<unsigned int>('\n')] = true;
 	if (!onlyLineEnds) {
@@ -63,7 +60,7 @@ static std::vector<char *> ArrayFromStringList(char *stringList, bool onlyLineEn
 		wordSeparator[static_cast<unsigned int>('\t')] = true;
 	}
 	for (int j = 0; stringList[j]; j++) {
-		int curr = static_cast<unsigned char>(stringList[j]);
+		const int curr = static_cast<unsigned char>(stringList[j]);
 		if (!wordSeparator[curr] && wordSeparator[prev])
 			words++;
 		prev = curr;
