@@ -9,11 +9,11 @@
 // It is provided "as is" without express or implied warranty.
 ////////////////////////////////////////////////////////////////////////////////
 
-#include "Utf8_16.h"
-
 #include <memory.h>
 #include <stdio.h>
 
+#include "Utf8_16.h"
+
 const Utf8_16::utf8 Utf8_16::k_Boms[][3] = {
 	{0x00, 0x00, 0x00},  // Unknown
 	{0xFE, 0xFF, 0x00},  // Big endian
@@ -198,10 +198,10 @@ size_t Utf8_16_Write::fwrite(const void* p, size_t _size) {
 			int codePoint = iter8.get();
 			if (codePoint >= SURROGATE_FIRST_VALUE) {
 				codePoint -= SURROGATE_FIRST_VALUE;
-				int lead = (codePoint >> 10) + SURROGATE_LEAD_FIRST;
+				const int lead = (codePoint >> 10) + SURROGATE_LEAD_FIRST;
 				*pCur++ = static_cast<utf16>((m_eEncoding == eUtf16BigEndian) ?
 					swapped(lead) : lead);
-				int trail = (codePoint & 0x3ff) + SURROGATE_TRAIL_FIRST;
+				const int trail = (codePoint & 0x3ff) + SURROGATE_TRAIL_FIRST;
 				*pCur++ = static_cast<utf16>((m_eEncoding == eUtf16BigEndian) ?
 					swapped(trail) : trail);
 			} else {
@@ -211,17 +211,21 @@ size_t Utf8_16_Write::fwrite(const void* p, size_t _size) {
 		}
 	}
 
-	size_t ret = ::fwrite(m_pBuf, (const char*)pCur - (const char*)m_pBuf, 1, m_pFile);
+	size_t ret = ::fwrite(m_pBuf,
+		reinterpret_cast<const char*>(pCur) - reinterpret_cast<const char*>(m_pBuf),
+		1, m_pFile);
 
 	return ret;
 }
 
-void Utf8_16_Write::fclose() {
+int Utf8_16_Write::fclose() {
 	delete [] m_pBuf;
 	m_pBuf = NULL;
 
-	::fclose(m_pFile);
+	const int ret = ::fclose(m_pFile);
 	m_pFile = NULL;
+
+	return ret;
 }
 
 void Utf8_16_Write::setEncoding(Utf8_16::encodingType eType) {
@@ -311,7 +315,7 @@ void Utf16_Iter::set
 	m_pEnd = pBuf + nLen;
 	m_eEncoding = eEncoding;
 	if (nLen > 2) {
-		utf16 lastElement = read(m_pEnd-2);
+		const utf16 lastElement = read(m_pEnd-2);
 		if (lastElement >= SURROGATE_LEAD_FIRST && lastElement <= SURROGATE_LEAD_LAST) {
 			// Buffer ends with lead surrogate so cut off buffer and store
 			endSurrogate[0] = m_pEnd[-2];
