@@ -1,9 +1,11 @@
 // Unit Tests for Scintilla internal data structures
 
-#include <string.h>
+#include <cstring>
 
 #include <stdexcept>
+#include <vector>
 #include <algorithm>
+#include <memory>
 
 #include "Platform.h"
 
@@ -116,7 +118,7 @@ TEST_CASE("RunStyles") {
 		int startFill2 = 2;
 		int lengthFill2 = 1;
 		// Compiler warnings if 'false' used instead of '0' as expected value:
-		REQUIRE(0 == rs.FillRange(startFill2, 99, lengthFill2));
+		REQUIRE(false == rs.FillRange(startFill2, 99, lengthFill2));
 		REQUIRE(2 == startFill2);
 		REQUIRE(1 == lengthFill2);
 		REQUIRE(0 == rs.ValueAt(0));
@@ -202,17 +204,17 @@ TEST_CASE("RunStyles") {
 		REQUIRE(true == rs.AllSame());
 		rs.InsertSpace(0, 5);
 		REQUIRE(true == rs.AllSame());
-		REQUIRE(0 == rs.AllSameAs(88));
+		REQUIRE(false == rs.AllSameAs(88));
 		REQUIRE(true == rs.AllSameAs(0));
 		int startFill = 1;
 		int lengthFill = 3;
 		REQUIRE(true == rs.FillRange(startFill, 99, lengthFill));
-		REQUIRE(0 == rs.AllSame());
-		REQUIRE(0 == rs.AllSameAs(88));
-		REQUIRE(0 == rs.AllSameAs(0));
+		REQUIRE(false == rs.AllSame());
+		REQUIRE(false == rs.AllSameAs(88));
+		REQUIRE(false == rs.AllSameAs(0));
 		REQUIRE(true == rs.FillRange(startFill, 0, lengthFill));
 		REQUIRE(true == rs.AllSame());
-		REQUIRE(0 == rs.AllSameAs(88));
+		REQUIRE(false == rs.AllSameAs(88));
 		REQUIRE(true == rs.AllSameAs(0));
 	}
 
