@@ -6,6 +6,7 @@
 // Copyright 1998-2010 by Neil Hodgson <neilh@scintilla.org>
 // The License.txt file describes the conditions under which this software may be distributed.
 
+#include <stdlib.h>
 #include <stdio.h>
 #include <time.h>
 
@@ -33,7 +34,7 @@ enum { SURROGATE_TRAIL_LAST = 0xDFFF };
 static unsigned int UTF8Length(const wchar_t *uptr, size_t tlen) {
 	unsigned int len = 0;
 	for (size_t i = 0; i < tlen && uptr[i];) {
-		unsigned int uch = uptr[i];
+		const unsigned int uch = uptr[i];
 		if (uch < 0x80) {
 			len++;
 		} else if (uch < 0x800) {
@@ -50,10 +51,10 @@ static unsigned int UTF8Length(const wchar_t *uptr, size_t tlen) {
 	return len;
 }
 
-static void UTF8FromUTF16(const wchar_t *uptr, size_t tlen, char *putf, size_t len) {
+static void UTF8FromUTF16(const wchar_t *uptr, size_t tlen, char *putf) {
 	int k = 0;
 	for (size_t i = 0; i < tlen && uptr[i];) {
-		unsigned int uch = uptr[i];
+		const unsigned int uch = uptr[i];
 		if (uch < 0x80) {
 			putf[k++] = static_cast<char>(uch);
 		} else if (uch < 0x800) {
@@ -63,7 +64,7 @@ static void UTF8FromUTF16(const wchar_t *uptr, size_t tlen, char *putf, size_t l
 			(uch <= SURROGATE_TRAIL_LAST)) {
 			// Half a surrogate pair
 			i++;
-			unsigned int xch = 0x10000 + ((uch & 0x3ff) << 10) + (uptr[i] & 0x3ff);
+			const unsigned int xch = 0x10000 + ((uch & 0x3ff) << 10) + (uptr[i] & 0x3ff);
 			putf[k++] = static_cast<char>(0xF0 | (xch >> 18));
 			putf[k++] = static_cast<char>(0x80 | ((xch >> 12) & 0x3f));
 			putf[k++] = static_cast<char>(0x80 | ((xch >> 6) & 0x3f));
@@ -75,14 +76,13 @@ static void UTF8FromUTF16(const wchar_t *uptr, size_t tlen, char *putf, size_t l
 		}
 		i++;
 	}
-	putf[len] = '\0';
 }
 
-static size_t UTF16Length(const char *s, unsigned int len) {
+static size_t UTF16Length(const char *s, size_t len) {
 	size_t ulen = 0;
 	size_t charLen;
 	for (size_t i=0; i<len;) {
-		unsigned char ch = static_cast<unsigned char>(s[i]);
+		const unsigned char ch = static_cast<unsigned char>(s[i]);
 		if (ch < 0x80) {
 			charLen = 1;
 		} else if (ch < 0x80 + 0x40 + 0x20) {
@@ -136,31 +136,35 @@ static size_t UTF16FromUTF8(const char *s, size_t len, gui_char *tbuf, size_t tl
 }
 
 gui_string StringFromUTF8(const char *s) {
-	if (!s) {
+	if (!s || !*s) {
 		return gui_string();
 	}
-	size_t sLen = strlen(s);
-	size_t wideLen = UTF16Length(s, static_cast<int>(sLen));
-	std::vector<gui_char> vgc(wideLen + 1);
-	size_t outLen = UTF16FromUTF8(s, sLen, &vgc[0], wideLen);
-	vgc[outLen] = 0;
-	return gui_string(&vgc[0], outLen);
+	const size_t sLen = strlen(s);
+	const size_t wideLen = UTF16Length(s, sLen);
+	std::vector<gui_char> vgc(wideLen);
+	UTF16FromUTF8(s, sLen, &vgc[0], wideLen);
+	return gui_string(&vgc[0], wideLen);
 }
 
 gui_string StringFromUTF8(const std::string &s) {
-	size_t sLen = s.length();
-	size_t wideLen = UTF16Length(s.c_str(), static_cast<int>(sLen));
-	std::vector<gui_char> vgc(wideLen + 1);
-	size_t outLen = UTF16FromUTF8(s.c_str(), sLen, &vgc[0], wideLen);
-	vgc[outLen] = 0;
-	return gui_string(&vgc[0], outLen);
+	if (s.empty()) {
+		return gui_string();
+	}
+	const size_t sLen = s.length();
+	const size_t wideLen = UTF16Length(s.c_str(), sLen);
+	std::vector<gui_char> vgc(wideLen);
+	UTF16FromUTF8(s.c_str(), sLen, &vgc[0], wideLen);
+	return gui_string(&vgc[0], wideLen);
 }
 
 std::string UTF8FromString(const gui_string &s) {
-	size_t sLen = s.size();
-	size_t narrowLen = UTF8Length(s.c_str(), sLen);
-	std::vector<char> vc(narrowLen + 1);
-	UTF8FromUTF16(s.c_str(), sLen, &vc[0], narrowLen);
+	if (s.empty()) {
+		return std::string();
+	}
+	const size_t sLen = s.size();
+	const size_t narrowLen = UTF8Length(s.c_str(), sLen);
+	std::vector<char> vc(narrowLen);
+	UTF8FromUTF16(s.c_str(), sLen, &vc[0]);
 	return std::string(&vc[0], narrowLen);
 }
 
@@ -296,12 +300,12 @@ double ElapsedTime::Duration(bool reset) {
 		LARGE_INTEGER lBegin;
 		lBegin.HighPart = bigBit;
 		lBegin.LowPart = littleBit;
-		double elapsed = static_cast<double>(lEnd.QuadPart - lBegin.QuadPart);
+		const double elapsed = static_cast<double>(lEnd.QuadPart - lBegin.QuadPart);
 		result = elapsed / static_cast<double>(frequency.QuadPart);
 	} else {
 		endBigBit = clock();
 		endLittleBit = 0;
-		double elapsed = endBigBit - bigBit;
+		const double elapsed = endBigBit - bigBit;
 		result = elapsed / CLOCKS_PER_SEC;
 	}
 	if (reset) {
