@@ -68,18 +68,18 @@ long SciTEKeys::ParseKeyCode(const char *mnemonic) {
 		} else if (sKey.length() > 1) {
 			if ((sKey.at(0) == 'F') && (isdigit(sKey.at(1)))) {
 				sKey.erase(0, 1);
-				int fkeyNum = atoi(sKey.c_str());
+				const int fkeyNum = atoi(sKey.c_str());
 				if (fkeyNum >= 1 && fkeyNum <= 12)
 					keyval = fkeyNum - 1 + VK_F1;
 			} else if ((sKey.at(0) == 'V') && (isdigit(sKey.at(1)))) {
 				sKey.erase(0, 1);
-				int vkey = atoi(sKey.c_str());
+				const int vkey = atoi(sKey.c_str());
 				if (vkey > 0 && vkey <= 0x7FFF)
 					keyval = vkey;
 			} else if (StartsWith(sKey, "Keypad")) {
 				sKey.erase(0, strlen("Keypad"));
 				if ((sKey.length() > 0) && isdigit(sKey.at(0))) {
-					int keyNum = atoi(sKey.c_str());
+					const int keyNum = atoi(sKey.c_str());
 					if (keyNum >= 0 && keyNum <= 9)
 						keyval = keyNum + VK_NUMPAD0;
 				} else if (sKey == "Plus") {
@@ -141,6 +141,32 @@ const TCHAR *SciTEWin::className = NULL;
 const TCHAR *SciTEWin::classNameInternal = NULL;
 SciTEWin *SciTEWin::app = NULL;
 
+namespace {
+
+// Using VerifyVersionInfo on Windows 10 will pretend its Windows 8
+// but that is good enough for switching UI elements to flatter.
+// The VersionHelpers.h functions can't be used as they aren't supported by GCC.
+
+bool UIShouldBeFlat() {
+	OSVERSIONINFOEX osvi = OSVERSIONINFOEX();
+	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
+	osvi.dwMajorVersion = 6;
+	osvi.dwMinorVersion = 2;
+
+	const BYTE op = VER_GREATER_EQUAL;
+	DWORDLONG dwlConditionMask = 0;
+	VER_SET_CONDITION(dwlConditionMask, VER_MAJORVERSION, op);
+	VER_SET_CONDITION(dwlConditionMask, VER_MINORVERSION, op);
+
+	return VerifyVersionInfo(
+		&osvi,
+		VER_MAJORVERSION | VER_MINORVERSION |
+		VER_SERVICEPACKMAJOR | VER_SERVICEPACKMINOR,
+		dwlConditionMask);
+}
+
+}
+
 SciTEWin::SciTEWin(Extension *ext) : SciTEBase(ext) {
 	app = this;
 
@@ -154,6 +180,8 @@ SciTEWin::SciTEWin(Extension *ext) : SciTEBase(ext) {
 	replaceStrip.SetLocalizer(&localiser);
 	replaceStrip.SetSearcher(this);
 
+	flatterUI = UIShouldBeFlat();
+
 	cmdShow = 0;
 	heightBar = 7;
 	fontTabs = 0;
@@ -272,11 +300,10 @@ static int CodePageFromName(const std::string &encodingName) {
 		{ "iso8859-11", 874 },
 		{ "1250", 1250 },
 		{ "windows-1251", 1251 },
-		{ 0, 0 },
 	};
-	for (Encoding *enc=knownEncodings; enc->name; enc++) {
-		if (encodingName == enc->name) {
-			return enc->codePage;
+	for (const Encoding &enc : knownEncodings) {
+		if (encodingName == enc.name) {
+			return enc.codePage;
 		}
 	}
 	return CP_UTF8;
@@ -319,7 +346,7 @@ void SciTEWin::ReadLocalization() {
 	std::string encoding = localiser.GetString("translation.encoding");
 	LowerCaseAZ(encoding);
 	if (encoding.length()) {
-		int codePageNamed = CodePageFromName(encoding);
+		const int codePageNamed = CodePageFromName(encoding);
 		const char *key = NULL;
 		const char *val = NULL;
 		// Get encoding
@@ -408,13 +435,20 @@ void SciTEWin::ReadEmbeddedProperties() {
 void SciTEWin::ReadPropertiesInitial() {
 	SciTEBase::ReadPropertiesInitial();
 	if (tabMultiLine) {	// Windows specific!
-		long wl = ::GetWindowLong(HwndOf(wTabBar), GWL_STYLE);
+		const long wl = ::GetWindowLong(HwndOf(wTabBar), GWL_STYLE);
 		::SetWindowLong(HwndOf(wTabBar), GWL_STYLE, wl | TCS_MULTILINE);
 	}
 }
 
 void SciTEWin::ReadProperties() {
 	SciTEBase::ReadProperties();
+	if (flatterUI) {
+		if (foldColour.empty() && foldHiliteColour.empty()) {
+			Colour lightMargin = ColourRGB(0xF7, 0xF7, 0xF7);
+			CallChildren(SCI_SETFOLDMARGINCOLOUR, 1, lightMargin);
+			CallChildren(SCI_SETFOLDMARGINHICOLOUR, 1, lightMargin);
+		}
+	}
 }
 
 static FilePath GetSciTEPath(const FilePath &home) {
@@ -617,10 +651,8 @@ void SciTEWin::Command(WPARAM wParam, LPARAM lParam) {
 	if (!menuSource) {
 		if (!wEditor.HasFocus() && !wOutput.HasFocus()) {
 			HWND wWithFocus = ::GetFocus();
-			enum { capSize = 2000 };
-			GUI::gui_char classNameFocus[capSize];
-			::GetClassName(wWithFocus, classNameFocus, capSize);
-			if (wcscmp(classNameFocus, TEXT("Edit")) == 0) {
+			const GUI::gui_string classNameFocus = ClassNameOfWindow(wWithFocus);
+			if (classNameFocus == GUI_TEXT("Edit")) {
 				switch (cmdID) {
 				case IDM_UNDO:
 					::SendMessage(wWithFocus, EM_UNDO, 0, 0);
@@ -667,7 +699,7 @@ void SciTEWin::Command(WPARAM wParam, LPARAM lParam) {
 		break;
 
 	case IDC_TABCLOSE:
-		CloseTab((int)lParam);
+		CloseTab(static_cast<int>(lParam));
 		break;
 
 	case IDC_SHIFTTAB:
@@ -682,7 +714,7 @@ void SciTEWin::Command(WPARAM wParam, LPARAM lParam) {
 // from ScintillaWin.cxx
 static UINT CodePageFromCharSet(DWORD characterSet, UINT documentCodePage) {
 	CHARSETINFO ci = { 0, 0, { { 0, 0, 0, 0 }, { 0, 0 } } };
-	BOOL bci = ::TranslateCharsetInfo(reinterpret_cast<DWORD*>(static_cast<uptr_t>(characterSet)),
+	const BOOL bci = ::TranslateCharsetInfo(reinterpret_cast<DWORD*>(static_cast<uptr_t>(characterSet)),
 	                                  &ci, TCI_SRCCHARSET);
 
 	UINT cp;
@@ -943,7 +975,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 
 				DWORD bytesWrote = 0;
 
-				int bTest = ::WriteFile(hWriteSubProcess,
+				const int bTest = ::WriteFile(hWriteSubProcess,
 					    jobToRun.input.c_str() + writingPosition,
 					    static_cast<DWORD>(bytesToWrite), &bytesWrote, NULL);
 
@@ -968,7 +1000,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 				}
 
 			} else if (bytesAvail > 0) {
-				int bTest = ::ReadFile(hPipeRead, &buffer[0],
+				const int bTest = ::ReadFile(hPipeRead, &buffer[0],
 						       static_cast<DWORD>(buffer.size()), &bytesRead, NULL);
 
 				if (bTest && bytesRead) {
@@ -1263,7 +1295,7 @@ static void WorkerThread(void *ptr) {
 }
 
 bool SciTEWin::PerformOnNewThread(Worker *pWorker) {
-	uintptr_t result = _beginthread(WorkerThread, 1024 * 1024, static_cast<void *>(pWorker));
+	const uintptr_t result = _beginthread(WorkerThread, 1024 * 1024, static_cast<void *>(pWorker));
 	return result != static_cast<uintptr_t>(-1);
 }
 
@@ -1320,7 +1352,7 @@ void SciTEWin::CreateUI() {
 	CreateBuffers();
 
 	int left = props.GetInt("position.left", CW_USEDEFAULT);
-	int top = props.GetInt("position.top", CW_USEDEFAULT);
+	const int top = props.GetInt("position.top", CW_USEDEFAULT);
 	int width = props.GetInt("position.width", CW_USEDEFAULT);
 	int height = props.GetInt("position.height", CW_USEDEFAULT);
 	cmdShow = props.GetInt("position.maximize", 0) ? SW_MAXIMIZE : 0;
@@ -1432,7 +1464,7 @@ void SciTEWin::Run(const GUI::gui_char *cmdLine) {
 	// we still get a last chance to force checking or to open a separate instance;
 	// Check if the user just want to print the file(s).
 	// Don't process files yet.
-	bool bBatchProcessing = ProcessCommandLine(args, 0);
+	const bool bBatchProcessing = ProcessCommandLine(args, 0);
 
 	// No need to check for other instances when doing a batch job:
 	// perform some tasks and exit immediately.
@@ -1484,24 +1516,32 @@ void SciTEWin::Run(const GUI::gui_char *cmdLine) {
  * Draw the split bar.
  */
 void ContentWin::Paint(HDC hDC, GUI::Rectangle) {
-	GUI::Rectangle rcInternal = GetClientPosition();
+	const GUI::Rectangle rcInternal = GetClientPosition();
 
-	int heightClient = rcInternal.Height();
-	int widthClient = rcInternal.Width();
+	const int heightClient = rcInternal.Height();
+	const int widthClient = rcInternal.Width();
 
-	int heightEditor = heightClient - pSciTEWin->heightOutput - pSciTEWin->heightBar;
-	int yBorder = heightEditor;
-	int xBorder = widthClient - pSciTEWin->heightOutput - pSciTEWin->heightBar;
+	const int heightEditor = heightClient - pSciTEWin->heightOutput - pSciTEWin->heightBar;
+	const int yBorder = heightEditor;
+	const int xBorder = widthClient - pSciTEWin->heightOutput - pSciTEWin->heightBar;
 	for (int i = 0; i < pSciTEWin->heightBar; i++) {
-		HPEN pen;
-		if (i == 1)
-			pen = ::CreatePen(0,1,::GetSysColor(COLOR_3DHIGHLIGHT));
-		else if (i == pSciTEWin->heightBar - 2)
-			pen = ::CreatePen(0,1,::GetSysColor(COLOR_3DSHADOW));
-		else if (i == pSciTEWin->heightBar - 1)
-			pen = ::CreatePen(0,1,::GetSysColor(COLOR_3DDKSHADOW));
-		else
-			pen = ::CreatePen(0,1,::GetSysColor(COLOR_3DFACE));
+		int colourIndex = COLOR_3DFACE;
+		if (pSciTEWin->flatterUI) {
+			if (i == 0 || i == pSciTEWin->heightBar - 1)
+				colourIndex = COLOR_3DFACE;
+			else
+				colourIndex = COLOR_WINDOW;
+		} else {
+			if (i == 1)
+				colourIndex = COLOR_3DHIGHLIGHT;
+			else if (i == pSciTEWin->heightBar - 2)
+				colourIndex = COLOR_3DSHADOW;
+			else if (i == pSciTEWin->heightBar - 1)
+				colourIndex = COLOR_3DDKSHADOW;
+			else
+				colourIndex = COLOR_3DFACE;
+		}
+		HPEN pen = ::CreatePen(0, 1, ::GetSysColor(colourIndex));
 		HPEN penOld = static_cast<HPEN>(::SelectObject(hDC, pen));
 		if (pSciTEWin->splitVertical) {
 			::MoveToEx(hDC, xBorder + i, 0, 0);
@@ -1530,11 +1570,11 @@ void SciTEWin::DropFiles(HDROP hdrop) {
 	// If drag'n'drop inside the SciTE window but outside
 	// Scintilla, hdrop is null, and an exception is generated!
 	if (hdrop) {
-		bool tempFilesSyncLoad = props.GetInt("temp.files.sync.load") != 0;
+		const bool tempFilesSyncLoad = props.GetInt("temp.files.sync.load") != 0;
 		GUI::gui_char tempDir[MAX_PATH];
 		DWORD tempDirLen = ::GetTempPath(MAX_PATH, tempDir);
 		bool isTempFile = false;
-		int filesDropped = ::DragQueryFile(hdrop, 0xffffffff, NULL, 0);
+		const int filesDropped = ::DragQueryFile(hdrop, 0xffffffff, NULL, 0);
 		// Append paths to dropFilesQueue, to finish drag operation soon
 		for (int i = 0; i < filesDropped; ++i) {
 			GUI::gui_char pathDropped[MAX_PATH];
@@ -1579,7 +1619,7 @@ bool SciTEWin::PreOpenCheck(const GUI::gui_char *arg) {
 	bool isHandled = false;
 	HANDLE hFFile;
 	WIN32_FIND_DATA ffile;
-	DWORD fileattributes = ::GetFileAttributes(arg);
+	const DWORD fileattributes = ::GetFileAttributes(arg);
 	int nbuffers = props.GetInt("buffers");
 	FilePath fpArg(arg);
 
@@ -1753,9 +1793,9 @@ LRESULT SciTEWin::KeyDown(WPARAM wParam) {
 
 	// loop through the keyboard short cuts defined by user.. if found
 	// exec it the command defined
-	for (size_t cut_i = 0; cut_i < shortCutItemList.size(); cut_i++) {
-		if (KeyMatch(shortCutItemList[cut_i].menuKey, static_cast<int>(wParam), modifiers)) {
-			int commandNum = SciTEBase::GetMenuCommandAsInt(shortCutItemList[cut_i].menuCommand.c_str());
+	for (const ShortcutItem &scut : shortCutItemList) {
+		if (KeyMatch(scut.menuKey, static_cast<int>(wParam), modifiers)) {
+			const int commandNum = SciTEBase::GetMenuCommandAsInt(scut.menuCommand.c_str());
 			if (commandNum != -1) {
 				// its possible that the command is for scintilla directly
 				// all scintilla commands are larger then 2000
@@ -2082,12 +2122,9 @@ LRESULT ContentWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 
 	case WM_SETCURSOR:
 		if (ControlIDOfCommand(static_cast<unsigned long>(lParam)) == HTCLIENT) {
-			GUI::Point ptCursor;
-			::GetCursorPos(PointPointer(&ptCursor));
-			GUI::Point ptClient = ptCursor;
-			::ScreenToClient(pSciTEWin->MainHWND(), PointPointer(&ptClient));
-			GUI::Rectangle rcScintilla = pSciTEWin->wEditor.GetPosition();
-			GUI::Rectangle rcOutput = pSciTEWin->wOutput.GetPosition();
+			const GUI::Point ptCursor = PointOfCursor();
+			const GUI::Rectangle rcScintilla = pSciTEWin->wEditor.GetPosition();
+			const GUI::Rectangle rcOutput = pSciTEWin->wOutput.GetPosition();
 			if (!rcScintilla.Contains(ptCursor) && !rcOutput.Contains(ptCursor)) {
 				::SetCursor(::LoadCursor(NULL, pSciTEWin->splitVertical ? IDC_SIZEWE : IDC_SIZENS));
 				return TRUE;
@@ -2137,7 +2174,7 @@ uptr_t SciTEWin::EventLoop() {
 	BOOL going = true;
 	while (going) {
 		if (needIdle) {
-			BOOL haveMessage = PeekMessageW(&msg, NULL, 0, 0, PM_NOREMOVE);
+			const BOOL haveMessage = PeekMessageW(&msg, NULL, 0, 0, PM_NOREMOVE);
 			if (!haveMessage) {
 				OnIdle();
 				continue;
@@ -2161,18 +2198,40 @@ uptr_t SciTEWin::EventLoop() {
 #pragma warning(disable: 28251)
 #endif
 
-int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, int) {
+#ifndef LOAD_LIBRARY_SEARCH_APPLICATION_DIR
+#define LOAD_LIBRARY_SEARCH_APPLICATION_DIR 0x200
+#endif
+#ifndef LOAD_LIBRARY_SEARCH_SYSTEM32
+#define LOAD_LIBRARY_SEARCH_SYSTEM32 0x800
+#endif
 
-	typedef BOOL (WINAPI *SetDllDirectorySig)(LPCTSTR lpPathName);
+static void RestrictDLLPath() {
+	// Try to limit the locations where DLLs will be loaded from to prevent binary planting.
+	// That is where a bad DLL is placed in the current directory or in the PATH.
+	typedef BOOL(WINAPI *SetDefaultDllDirectoriesSig)(DWORD DirectoryFlags);
+	typedef BOOL(WINAPI *SetDllDirectorySig)(LPCTSTR lpPathName);
 	HMODULE kernel32 = ::GetModuleHandle(TEXT("kernel32.dll"));
 	if (kernel32) {
-		SetDllDirectorySig SetDllDirectoryFn = (SetDllDirectorySig)::GetProcAddress(
-			kernel32, "SetDllDirectoryW");
-		if (SetDllDirectoryFn) {
-			// For security, remove current directory from the DLL search path
-			SetDllDirectoryFn(TEXT(""));
+		// SetDefaultDllDirectories is stronger, limiting search path to just the application and
+		// system directories but is only available on Windows 8+
+		SetDefaultDllDirectoriesSig SetDefaultDllDirectoriesFn = (SetDefaultDllDirectoriesSig)::GetProcAddress(
+			kernel32, "SetDefaultDllDirectories");
+		if (SetDefaultDllDirectoriesFn) {
+			SetDefaultDllDirectoriesFn(LOAD_LIBRARY_SEARCH_APPLICATION_DIR | LOAD_LIBRARY_SEARCH_SYSTEM32);
+		} else {
+			SetDllDirectorySig SetDllDirectoryFn = (SetDllDirectorySig)::GetProcAddress(
+				kernel32, "SetDllDirectoryW");
+			if (SetDllDirectoryFn) {
+				// For security, remove current directory from the DLL search path
+				SetDllDirectoryFn(TEXT(""));
+			}
 		}
 	}
+}
+
+int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, int) {
+
+	RestrictDLLPath();
 
 #ifndef NO_EXTENSIONS
 	MultiplexExtension multiExtender;
@@ -2226,6 +2285,7 @@ int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, int) {
 		} catch (GUI::ScintillaFailure &sf) {
 			MainWind.CheckForScintillaFailure(static_cast<int>(sf.status));
 		}
+		MainWind.Finalise();
 	}
 
 #ifdef STATIC_BUILD
