@@ -35,7 +35,7 @@ void SciTEWin::SetFileProperties(
 		                NULL, temp, TEMP_LEN);
 		ps.Set("FileDate", temp);
 
-		DWORD attr = ::GetFileAttributesW(filePath.AsInternal());
+		const DWORD attr = ::GetFileAttributesW(filePath.AsInternal());
 		std::string fa;
 		if (attr & FILE_ATTRIBUTE_READONLY) {
 			fa += "R";
@@ -80,27 +80,50 @@ void SciTEWin::TabInsert(int index, const GUI::gui_char *title) {
 	tie.iImage = -1;
 	GUI::gui_string titleCopy(title, title + wcslen(title) + 1);
 	tie.pszText = &titleCopy[0];
-	::SendMessage(HwndOf(wTabBar), TCM_INSERTITEMW, index, reinterpret_cast<LPARAM>(&tie));
+	TabCtrl_InsertItem(HwndOf(wTabBar), index, &tie);
 }
 
 void SciTEWin::TabSelect(int index) {
-	::SendMessage(HwndOf(wTabBar), TCM_SETCURSEL, index, 0);
+	TabCtrl_SetCurSel(HwndOf(wTabBar), index);
 }
 
 void SciTEWin::RemoveAllTabs() {
-	::SendMessage(HwndOf(wTabBar), TCM_DELETEALLITEMS, 0, 0);
+	TabCtrl_DeleteAllItems(HwndOf(wTabBar));
+}
+
+GUI::Point PointOfCursor() {
+	POINT ptCursor;
+	::GetCursorPos(&ptCursor);
+	return GUI::Point(ptCursor.x, ptCursor.y);
+}
+
+GUI::Point ClientFromScreen(HWND hWnd, GUI::Point ptScreen) {
+	POINT ptClient = { ptScreen.x, ptScreen.y };
+	::ScreenToClient(hWnd, &ptClient);
+	return GUI::Point(ptClient.x, ptClient.y);
+}
+
+namespace {
+
+int TabAtPoint(HWND hWnd, GUI::Point pt) {
+	TCHITTESTINFO thti;
+	thti.pt.x = pt.x;
+	thti.pt.y = pt.y;
+	thti.flags = 0;
+	return TabCtrl_HitTest(hWnd, &thti);
+}
+
 }
 
 /**
  * Manage Windows specific notifications.
  */
-void SciTEWin::Notify(const SCNotification *notification) {
+void SciTEWin::Notify(SCNotification *notification) {
 	switch (notification->nmhdr.code) {
 	case TCN_SELCHANGE:
 		// Change of tab
 		if (notification->nmhdr.idFrom == IDM_TABWIN) {
-			int index = static_cast<int>(
-				::SendMessage(HwndOf(wTabBar), TCM_GETCURSEL, 0, 0));
+			const int index = TabCtrl_GetCurSel(HwndOf(wTabBar));
 			SetDocumentAt(index);
 			CheckReload();
 		}
@@ -110,15 +133,9 @@ void SciTEWin::Notify(const SCNotification *notification) {
 		// Right click on a control
 		if (notification->nmhdr.idFrom == IDM_TABWIN) {
 
-			GUI::Point ptCursor;
-			::GetCursorPos(PointPointer(&ptCursor));
-			GUI::Point ptClient = ptCursor;
-			::ScreenToClient(HwndOf(wTabBar), PointPointer(&ptClient));
-			TCHITTESTINFO info;
-			info.pt.x = ptClient.x;
-			info.pt.y = ptClient.y;
-
-			int tabbarHitLast = TabCtrl_HitTest(HwndOf(wTabBar), &info);
+			const GUI::Point ptCursor = PointOfCursor();
+			const GUI::Point ptClient = ClientFromScreen(HwndOf(wTabBar), ptCursor);
+			const int tabbarHitLast = TabAtPoint(HwndOf(wTabBar), ptClient);
 
 			if (buffers.Current() != tabbarHitLast) {
 				SetDocumentAt(tabbarHitLast);
@@ -158,7 +175,7 @@ void SciTEWin::Notify(const SCNotification *notification) {
 		// Click on a control
 		if (notification->nmhdr.idFrom == IDM_STATUSWIN) {
 			// Click on the status bar
-			NMMOUSE *pNMMouse = (NMMOUSE *)notification;
+			const NMMOUSE *pNMMouse = reinterpret_cast<NMMOUSE *>(notification);
 			switch (pNMMouse->dwItemSpec) {
 			case 0: 		/* Display of status */
 				sbNum++;
@@ -177,7 +194,7 @@ void SciTEWin::Notify(const SCNotification *notification) {
 		// Ask for tooltip text
 		{
 			const GUI::gui_char *ttext = 0;
-			NMTTDISPINFOW *pDispInfo = (NMTTDISPINFOW *)notification;
+			NMTTDISPINFOW *pDispInfo = reinterpret_cast<NMTTDISPINFOW *>(notification);
 			// Toolbar tooltips
 			switch (notification->nmhdr.idFrom) {
 			case IDM_NEW:
@@ -230,15 +247,8 @@ void SciTEWin::Notify(const SCNotification *notification) {
 				break;
 			default: {
 					// notification->nmhdr.idFrom appears to be the buffer number for tabbar tooltips
-					GUI::Point ptCursor;
-					::GetCursorPos(PointPointer(&ptCursor));
-					GUI::Point ptClient = ptCursor;
-					::ScreenToClient(HwndOf(wTabBar), PointPointer(&ptClient));
-					TCHITTESTINFO info;
-					info.pt.x = ptClient.x;
-					info.pt.y = ptClient.y;
-					int index = static_cast<int>(
-						::SendMessage(HwndOf(wTabBar), TCM_HITTEST, (WPARAM)0, (LPARAM) & info));
+					const GUI::Point ptClient = ClientFromScreen(HwndOf(wTabBar), PointOfCursor());
+					const int index = TabAtPoint(HwndOf(wTabBar), ptClient);
 					if (index >= 0) {
 						GUI::gui_string path = buffers.buffers[index].AsInternal();
 						// Handle '&' characters in path, since they are interpreted in
@@ -309,7 +319,7 @@ void SciTEWin::ActivateWindow(const char *) {
 enum { tickerID = 100 };
 
 void SciTEWin::TimerStart(int mask) {
-	int maskNew = timerMask | mask;
+	const int maskNew = timerMask | mask;
 	if (timerMask != maskNew) {
 		if (timerMask == 0) {
 			// Create a 1 second ticker
@@ -320,7 +330,7 @@ void SciTEWin::TimerStart(int mask) {
 }
 
 void SciTEWin::TimerEnd(int mask) {
-	int maskNew = timerMask & ~mask;
+	const int maskNew = timerMask & ~mask;
 	if (timerMask != maskNew) {
 		if (maskNew == 0) {
 			::KillTimer(HwndOf(wSciTE), tickerID);
@@ -366,7 +376,7 @@ void SciTEWin::SizeSubWindows() {
 
 	if (tabVisible) {	// ? hide one tab only
 		showTab = tabHideOne ?
-		          ::SendMessage(HwndOf(wTabBar), TCM_GETITEMCOUNT, 0, 0) > 1 :
+		          TabCtrl_GetItemCount(HwndOf(wTabBar)) > 1 :
 		          true;
 	}
 
@@ -378,8 +388,7 @@ void SciTEWin::SizeSubWindows() {
 	}
 
 	RECT r = { rcClient.left, 0, rcClient.right, 0 };
-	::SendMessage(HwndOf(wTabBar),
-		TCM_ADJUSTRECT, TRUE, LPARAM(&r));
+	TabCtrl_AdjustRect(HwndOf(wTabBar), TRUE, &r);
 	bands[bandTab].height = r.bottom - r.top - 4;
 
 	bands[bandBackground].visible = backgroundStrip.visible;
@@ -397,9 +406,9 @@ void SciTEWin::SizeSubWindows() {
 	if (heightContent <= 0)
 		heightContent = 1;
 
-	for (size_t i=0;i<bands.size();i++) {
-		if (bands[i].visible && !bands[i].expands)
-			heightContent -= bands[i].height;
+	for (const Band &band : bands) {
+		if (band.visible && !band.expands)
+			heightContent -= band.height;
 	}
 	if (heightContent <= 0) {
 		heightContent = rcClient.Height();
@@ -415,23 +424,20 @@ void SciTEWin::SizeSubWindows() {
 	HDWP hdwp = BeginDeferWindowPos(10);
 
 	int yPos = rcClient.top;
-	for (size_t b=0; b<bands.size(); b++) {
-		if (bands[b].visible) {
-			GUI::Rectangle rcToSet(rcClient.left, yPos, rcClient.right, yPos + bands[b].height);
+	for (const Band &band : bands) {
+		if (band.visible) {
+			GUI::Rectangle rcToSet(rcClient.left, yPos, rcClient.right, yPos + band.height);
 			if (hdwp)
-				hdwp = ::DeferWindowPos(hdwp, HwndOf(bands[b].win),
+				hdwp = ::DeferWindowPos(hdwp, HwndOf(band.win),
 				0, rcToSet.left, rcToSet.top, rcToSet.Width(), rcToSet.Height(),
 				SWP_NOZORDER|SWP_NOACTIVATE|SWP_SHOWWINDOW);
-			//bands[b].win.Show(true);
-			yPos += bands[b].height;
+			yPos += band.height;
 		} else {
 			GUI::Rectangle rcToSet(rcClient.left, rcClient.top - 41, rcClient.Width(), rcClient.top - 40);
 			if (hdwp)
-				hdwp = ::DeferWindowPos(hdwp, HwndOf(bands[b].win),
+				hdwp = ::DeferWindowPos(hdwp, HwndOf(band.win),
 				0, rcToSet.left, rcToSet.top, rcToSet.Width(), rcToSet.Height(),
 				SWP_NOZORDER|SWP_NOACTIVATE|SWP_HIDEWINDOW);
-			//bands[b].win.Show(false);
-			//bands[b].win.SetPosition();
 		}
 	}
 	if (hdwp)
@@ -471,7 +477,7 @@ void SciTEWin::SetMenuItem(int menuNumber, int position, int itemID,
 		// tools, and for other menu entries it is just discarded.
 	}
 
-	UINT typeFlags = (text[0]) ? MF_STRING : MF_SEPARATOR;
+	const UINT typeFlags = (text[0]) ? MF_STRING : MF_SEPARATOR;
 	if (::GetMenuState(hmenu, itemID, MF_BYCOMMAND) == (UINT)(-1)) {
 		// Not present so insert
 		::InsertMenuW(hmenu, position, MF_BYPOSITION | typeFlags, itemID, sTextMnemonic.c_str());
@@ -584,12 +590,10 @@ void SciTEWin::LocaliseMenus() {
 }
 
 void SciTEWin::LocaliseControl(HWND w) {
-	char wtext[200];
-	if (::GetWindowTextA(w, wtext, sizeof(wtext))) {
-		GUI::gui_string text = localiser.Text(wtext, false);
-		if (text.length())
-			::SetWindowTextW(w, text.c_str());
-	}
+	std::string originalText = GUI::UTF8FromString(TextOfWindow(w));
+	GUI::gui_string translatedText = localiser.Text(originalText.c_str(), false);
+	if (translatedText.length())
+		::SetWindowTextW(w, translatedText.c_str());
 }
 
 void SciTEWin::LocaliseDialog(HWND wDialog) {
@@ -647,11 +651,7 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 
 	case WM_LBUTTONDOWN: {
 			GUI::Point pt = PointFromLong(lParam);
-			TCHITTESTINFO thti;
-			thti.pt.x = pt.x;
-			thti.pt.y = pt.y;
-			thti.flags = 0;
-			st_iLastClickTab = static_cast<int>(::SendMessage(hWnd, TCM_HITTEST, (WPARAM)0, (LPARAM) & thti));
+			st_iLastClickTab = TabAtPoint(hWnd, pt);
 		}
 		break;
 	}
@@ -667,14 +667,10 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 
 	case WM_MBUTTONDOWN: {
 			// Check if on tab bar
-			GUI::Point pt = PointFromLong(lParam);
-			TCHITTESTINFO thti;
-			thti.pt.x = pt.x;
-			thti.pt.y = pt.y;
-			thti.flags = 0;
-			int tab = static_cast<int>(::SendMessage(hWnd, TCM_HITTEST, (WPARAM)0, (LPARAM) & thti));
+			const GUI::Point pt = PointFromLong(lParam);
+			const int tab = TabAtPoint(hWnd, pt);
 			if (tab >= 0) {
-				::SendMessage(::GetParent(hWnd), WM_COMMAND, IDC_TABCLOSE, (LPARAM)tab);
+				::SendMessage(::GetParent(hWnd), WM_COMMAND, IDC_TABCLOSE, tab);
 			}
 		}
 		break;
@@ -686,12 +682,8 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 				::ReleaseCapture();
 				::SetCursor(::LoadCursor(NULL, IDC_ARROW));
 				st_bDragBegin = FALSE;
-				GUI::Point pt = PointFromLong(lParam);
-				TCHITTESTINFO thti;
-				thti.pt.x = pt.x;
-				thti.pt.y = pt.y;
-				thti.flags = 0;
-				int tab = static_cast<int>(::SendMessage(hWnd, TCM_HITTEST, (WPARAM)0, (LPARAM) & thti));
+				const GUI::Point pt = PointFromLong(lParam);
+				const int tab = TabAtPoint(hWnd, pt);
 				if (tab > -1 && st_iDraggingTab > -1 && st_iDraggingTab != tab) {
 					::SendMessage(::GetParent(hWnd),
 					        WM_COMMAND,
@@ -720,13 +712,9 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 
 	case WM_MOUSEMOVE: {
 
-			GUI::Point pt = PointFromLong(lParam);
-			TCHITTESTINFO thti;
-			thti.pt.x = pt.x;
-			thti.pt.y = pt.y;
-			thti.flags = 0;
-			int tab = static_cast<int>(::SendMessage(hWnd, TCM_HITTEST, (WPARAM)0, (LPARAM) & thti));
-			int tabcount = static_cast<int>(::SendMessage(hWnd, TCM_GETITEMCOUNT, (WPARAM)0, (LPARAM)0));
+			const GUI::Point pt = PointFromLong(lParam);
+			const int tab = TabAtPoint(hWnd, pt);
+			const int tabcount = TabCtrl_GetItemCount(hWnd);
 
 			if (wParam == MK_LBUTTON &&
 			        tabcount > 1 &&
@@ -757,15 +745,8 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 	case WM_PAINT: {
 			if (st_bDragBegin == TRUE && st_iDraggingTab != -1) {
 
-				GUI::Point ptCursor;
-				::GetCursorPos(PointPointer(&ptCursor));
-				GUI::Point ptClient = ptCursor;
-				::ScreenToClient(hWnd, PointPointer(&ptClient));
-				TCHITTESTINFO thti;
-				thti.pt.x = ptClient.x;
-				thti.pt.y = ptClient.y;
-				thti.flags = 0;
-				int tab = static_cast<int>(::SendMessage(hWnd, TCM_HITTEST, (WPARAM)0, (LPARAM) & thti));
+				const GUI::Point ptClient = ClientFromScreen(hWnd, PointOfCursor());
+				const int tab = TabAtPoint(hWnd, ptClient);
 
 				RECT tabrc;
 				if (tab != -1 &&
@@ -826,7 +807,7 @@ static LRESULT PASCAL TabWndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM
 void SciTEWin::Creation() {
 
 	wContent = ::CreateWindowEx(
-	               WS_EX_CLIENTEDGE,
+	               flatterUI ? 0 : WS_EX_CLIENTEDGE,
 	               classNameInternal,
 	               TEXT("Source"),
 	               WS_CHILD | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
@@ -901,7 +882,7 @@ void SciTEWin::Creation() {
 		addbmp.nID = IDR_CLOSEFILE24;
 	}
 	
-	::SendMessage(hwndToolBar, TB_ADDBITMAP, 1, (LPARAM)&addbmp);
+	::SendMessage(hwndToolBar, TB_ADDBITMAP, 1, reinterpret_cast<LPARAM>(&addbmp));
 
 	TBBUTTON tbb[ELEMENTS(bbs)];
 	for (unsigned int i = 0;i < ELEMENTS(bbs);i++) {
