@@ -20,7 +20,7 @@ static void FlashThisWindow(
 	if (hDC != NULL) {
 		RECT rc;
 		::GetClientRect(hWnd, &rc);
-		::FillRect(hDC, &rc, (HBRUSH)GetStockObject(BLACK_BRUSH));
+		::FillRect(hDC, &rc, static_cast<HBRUSH>(GetStockObject(BLACK_BRUSH)));
 		::Sleep(duration);
 		::ReleaseDC(hWnd, hDC);
 	}
@@ -114,7 +114,7 @@ void SciTEWin::WarnUser(int warnID) {
 	next = GetNextPropItem(next, sound, _MAX_PATH);
 	GetNextPropItem(next, soundDuration, 10);
 
-	int flashLen = atoi(flashDuration);
+	const int flashLen = atoi(flashDuration);
 	if (flashLen) {
 		FlashThisWindow(HwndOf(wEditor), flashLen);
 	}
@@ -139,7 +139,7 @@ bool SciTEWin::ModelessHandler(MSG *pmsg) {
 	if (wParameters.GetID()) {
 		// Allow commands, such as Ctrl+1 to be active while the Parameters dialog is
 		// visible so that a group of commands can be easily run with differing parameters.
-		bool menuKey = (pmsg->message == WM_KEYDOWN) &&
+		const bool menuKey = (pmsg->message == WM_KEYDOWN) &&
 		               (pmsg->wParam != VK_TAB) &&
 		               (pmsg->wParam != VK_ESCAPE) &&
 		               (pmsg->wParam != VK_RETURN) &&
@@ -171,7 +171,7 @@ bool SciTEWin::ModelessHandler(MSG *pmsg) {
 
 //  DoDialog is a bit like something in PC Magazine May 28, 1991, page 357
 int SciTEWin::DoDialog(const TCHAR *resName, DLGPROC lpProc) {
-	int result = static_cast<int>(
+	const int result = static_cast<int>(
 		::DialogBoxParam(hInstance, resName, MainHWND(), lpProc, reinterpret_cast<LPARAM>(this)));
 
 	if (result == -1) {
@@ -219,7 +219,7 @@ GUI::gui_string SciTEWin::DialogFilterFromProperty(const GUI::gui_char *filterPr
 }
 
 void SciTEWin::CheckCommonDialogError() {
-	DWORD errorNumber = ::CommDlgExtendedError();
+	const DWORD errorNumber = ::CommDlgExtendedError();
 	if (errorNumber) {
 		GUI::gui_string sError = GUI::HexStringFromInteger(errorNumber);
 		GUI::gui_string msg = LocaliseMessage("Common dialog error 0x^0.", sError.c_str());
@@ -546,11 +546,11 @@ void SciTEWin::Print(
 	// area of the page.
 
 	// Convert device coordinates into logical coordinates
-	DPtoLP(hdc, (LPPOINT) &rectMargins, 2);
-	DPtoLP(hdc, (LPPOINT)&rectPhysMargins, 2);
+	DPtoLP(hdc, reinterpret_cast<LPPOINT>(&rectMargins), 2);
+	DPtoLP(hdc, reinterpret_cast<LPPOINT>(&rectPhysMargins), 2);
 
 	// Convert page size to logical units and we're done!
-	DPtoLP(hdc, (LPPOINT) &ptPage, 1);
+	DPtoLP(hdc, reinterpret_cast<LPPOINT>(&ptPage), 1);
 
 	std::string headerFormat = props.GetString("print.header.format");
 	std::string footerFormat = props.GetString("print.footer.format");
@@ -780,14 +780,7 @@ public:
 	}
 
 	GUI::gui_string ItemTextG(int id) {
-		HWND wT = Item(id);
-		int len = ::GetWindowTextLengthW(wT) + 1;
-		std::vector<GUI::gui_char> itemText(len);
-		if (::GetDlgItemTextW(hDlg, id, &itemText[0], len)) {
-			return GUI::gui_string(&itemText[0]);
-		} else {
-			return GUI::gui_string();
-		}
+		return TextOfWindow(Item(id));
 	}
 
 	void SetItemText(int id, const GUI::gui_char *s) {
@@ -961,15 +954,15 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		} else if ( (ControlIDOfWParam(wParam) == IDOK) ||
 		            (ControlIDOfWParam(wParam) == IDMARKALL) ) {
 			dlg.GrabFields();
-			if (closeFind != Searcher::CloseFind::closePrevent) {
-				::EndDialog(hDlg, IDOK);
-				wFindReplace.Destroy();
-			}
-			if (ControlIDOfWParam(wParam) == IDMARKALL){
+			if (ControlIDOfWParam(wParam) == IDMARKALL) {
 				MarkAll(markWithBookMarks);
 			}
 			// Holding the Shift key inverts the current reverse flag
-			FindNext(reverseFind != IsKeyDown(VK_SHIFT));
+			const bool found = FindNext(reverseFind != IsKeyDown(VK_SHIFT)) >= 0;
+			if (ShouldClose(found)) {
+				::EndDialog(hDlg, IDOK);
+				wFindReplace.Destroy();
+			}
 			return TRUE;
 		} else if (ControlIDOfWParam(wParam) == IDFINDINSTYLE) {
 			if (FindReplaceAdvanced()) {
@@ -1378,8 +1371,8 @@ BOOL SciTEWin::GoLineMessage(HWND hDlg, UINT message, WPARAM wParam) {
 
 	case WM_INITDIALOG: {
 			int position = wEditor.Call(SCI_GETCURRENTPOS);
-			int lineNumber = wEditor.Call(SCI_LINEFROMPOSITION, position) + 1;
-			int lineStart = wEditor.Call(SCI_POSITIONFROMLINE, lineNumber - 1);
+			const int lineNumber = wEditor.Call(SCI_LINEFROMPOSITION, position) + 1;
+			const int lineStart = wEditor.Call(SCI_POSITIONFROMLINE, lineNumber - 1);
 			int characterOnLine = 1;
 			while (position > lineStart) {
 				position = wEditor.Call(SCI_POSITIONBEFORE, position);
@@ -1419,8 +1412,8 @@ BOOL SciTEWin::GoLineMessage(HWND hDlg, UINT message, WPARAM wParam) {
 
 				if (bHasChar && characterOnLine > 1 && lineNumber <= wEditor.Call(SCI_GETLINECOUNT)) {
 					// Constrain to the requested line
-					int lineStart = wEditor.Call(SCI_POSITIONFROMLINE, lineNumber - 1);
-					int lineEnd = wEditor.Call(SCI_GETLINEENDPOSITION, lineNumber - 1);
+					const int lineStart = wEditor.Call(SCI_POSITIONFROMLINE, lineNumber - 1);
+					const int lineEnd = wEditor.Call(SCI_GETLINEENDPOSITION, lineNumber - 1);
 
 					int position = lineStart;
 					while (--characterOnLine && position < lineEnd)
@@ -1490,16 +1483,13 @@ BOOL SciTEWin::TabSizeMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			int tabSize = wEditor.Call(SCI_GETTABWIDTH);
 			if (tabSize > 99)
 				tabSize = 99;
-			char tmp[3];
-			sprintf(tmp, "%d", tabSize);
-			::SetDlgItemTextA(hDlg, IDTABSIZE, tmp);
+			::SetDlgItemInt(hDlg, IDTABSIZE, tabSize, FALSE);
 
 			::SendDlgItemMessage(hDlg, IDINDENTSIZE, EM_LIMITTEXT, 2, 1);
 			int indentSize = wEditor.Call(SCI_GETINDENT);
 			if (indentSize > 99)
 				indentSize = 99;
-			sprintf(tmp, "%d", indentSize);
-			::SetDlgItemTextA(hDlg, IDINDENTSIZE, tmp);
+			::SetDlgItemInt(hDlg, IDINDENTSIZE, indentSize, FALSE);
 
 			::CheckDlgButton(hDlg, IDUSETABS, wEditor.Call(SCI_GETUSETABS));
 			return TRUE;
@@ -1631,7 +1621,7 @@ bool SciTEWin::ParametersDialog(bool modal) {
 
 SciTEBase::MessageBoxChoice SciTEWin::WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, MessageBoxStyle style) {
 	dialogsOnScreen++;
-	int ret = ::MessageBoxW(HwndOf(w), msg.c_str(), appName, style | MB_SETFOREGROUND);
+	const int ret = ::MessageBoxW(HwndOf(w), msg.c_str(), appName, style | MB_SETFOREGROUND);
 	dialogsOnScreen--;
 	switch (ret) {
 	case IDOK:
